# 'Standard Library'
from os import linesep as NL

from typing import Any, Dict, List


def ddir(
    thing_string: str,
    indent: int = 2,
    skip_dunders: bool = True,
    skip_builtins: bool = True,
    skip_join: bool = True,
) -> (List, str):
    result: List = []
    value: str = ""

    try:
        thing = eval(f"{thing_string}")
        thing
    except:
        raise

    thing = thing_string

    try:
        getattr(thing, __iter__)
        iter_thing = thing
    except:
        iter_thing = dir(thing)
    # print('iter_thing: ', iter_thing)
    for k in iter_thing:
        # print(f'{thing}.k: ', k)
        if skip_dunders and k.startswith("_"):
            continue
        if isinstance(k, (dict, list)):
            value = ddir(k)
        else:
            try:
                value = eval(k)
                if str(value).startswith("<func"):
                    if skip_builtins:
                        continue
                    value = f"<function {k}>"
                # assert False
            except:
                try:
                    evl = f"{thing_string}.{k}"
                    # print(evl)
                    value = eval(f"{evl}()")
                except:
                    try:
                        value = eval(f"{evl}")
                        # value = str(value)
                        # print("no parens eval: ", value)
                        if str(value).startswith("<built-in"):
                            if skip_builtins:
                                continue
                            value = f"<builtin: '{k}'>"
                    except:
                        try:
                            value = str(value)
                            print(value)

                        except:
                            pass
                        #     value = f"{k}"
                        #     if value.startswith("<meth"):
                        #         value = f"<method {k}>"
                        # except:
                        #     value = f"<{k}>"

        result.append(f"{indent*' '}{k:<15.15}: {value}")
        if not skip_join:
            result = NL.join(result)
    return result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" as_tempfile.py - temporary files and folders

    Part of the `AutoSys` package - utilities for macOS apps
        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """

# 'Standard Library'
import shutil
import sys
import tempfile

from dataclasses import Field, dataclass, field
from io import TextIOWrapper
from locale import getpreferredencoding
from os import linesep as NL
from pathlib import Path

# 'package imports'
from autosys.text_utils.nowandthen import now

from typing import Any, Deque, Dict, Final, List, Sequence, Set, Tuple

_debug_: Final[bool] = True
DEFAULT_ENCODING: Final[str] = getpreferredencoding(do_setlocale=True) or "utf-8"
copyright_symbol: Final[str] = "©"  # could be (c)

TMP_FD: TextIOWrapper = None



@dataclass
class TempFile:
    """ Temporary File Utility.
        Creates and manages temporary files, providing utilities for:

        - prefix
        - suffix
        - folder
        - delete when done?
        - named?
        - iterator?
        - generator?
        - log access?
        - cache access?
        -


        if name is provided, use name.
        if folder is provided, use folder.
        if delete?, use ...
        """

    fh: int = 0
    name: str = ""
    path: Path = field(default_factory=Path)
    log: bool = field(default=False)
    history: int = field(default=1000)

    def __post_init__(self):
        pass

    def __del__(self):
        pass
        # if not shutil.rm_tree():
        # del self

    def temp_it(self, func):
        def _temp_it_(self):
            with tempfile.NamedTemporaryFile as TMP_FD:
                result = func()
            return result

        return _temp_it_()

    def __enter__(self):
        return super().__enter__()

    def __exit__(self, *args):
        """
            This function only writes to disk file on exit. It is intended for use in short blocks of code as a temporary context manager. Long term usage increases the risk of data loss and will act as a memory leak.
            """
        # self.extend(self._stringio.getvalue().splitlines())
        # print(self.get_data(), file=sys.stderr)

        # self.truncate(0)
        # sys.stdout = self._stdout
        # del self  # free up some memory
        pass


tmp = TempFile()


@tmp.temp_it
def test_it():
    print(f"{temp_file=}")
    print(f"{TMP_FD}")


if __name__ == "__main__":

    # # fake 'with' block for directory
    # tmpdir = tempfile.mkdtemp()
    # print("temporary directory at " + tmpdir)
    # # do things with the tmpdir
    # shutil.rm_tree(tmpdir)

    print("testing decorated function ...")
    test_it()



#!/usr/bin/env python3
# !---------------------------------------------- Imports
if True:  # ! -- System Imports
    import platform
    import requests
    from subprocess import check_output
    import sys

if True:  # ! -- Specific System Imports
    # from base64 import b64decode, b64encode
    # from pathlib import Path
    # from typing import Dict, List
    # try:
    #     import ujson as json
    # except:
    #     import json
    pass
if True:  # ! -- Package Imports
    from autosys.utils import dbprint, read, NL


class DnsError(Exception):
    pass


def is_valid_ipv4_address(address):
    import socket

    try:
        socket.inet_pton(socket.AF_INET, address)
    except AttributeError:  # no inet_pton here, sorry
        try:
            socket.inet_aton(address)
        except socket.error:
            return False
        return address.count(".") == 3
    except socket.error:  # not a valid address
        return False

    return True


def get_unix_dns_ips():
    dns_ips = []
    # macOS /etc/resolv.conf is mapped from /var/run/resolv.conf
    # -rw-r--r-- 1 root daemon 484 Apr 14 07:10 resolv.conf
    with open("/etc/resolv.conf") as fp:
        for cnt, line in enumerate(fp):
            columns = line.split()
            if columns[0] == "nameserver":
                ip = columns[1:][0]
                if is_valid_ipv4_address(ip):
                    dns_ips.append(ip)
    return dns_ips


def get_windows_dns_ips():
    output = check_output(["ipconfig", "-all"])
    ipconfig_all_list = output.split("\n")

    dns_ips = []
    for i in range(0, len(ipconfig_all_list)):
        if "DNS Servers" in ipconfig_all_list[i]:
            # get the first dns server ip
            first_ip = ipconfig_all_list[i].split(":")[1].strip()
            if not is_valid_ipv4_address(first_ip):
                continue
            dns_ips.append(first_ip)
            # get all other dns server ips if they exist
            k = i + 1
            while (
                k < len(ipconfig_all_list) and ":" not in ipconfig_all_list[k]
            ):
                ip = ipconfig_all_list[k].strip()
                if is_valid_ipv4_address(ip):
                    dns_ips.append(ip)
                k += 1
            # at this point we're done
            break
    return dns_ips


def main():
    if platform.system() == "Windows":
        dns_ips = get_windows_dns_ips()
    elif platform.system() == "Darwin":
        dns_ips = get_unix_dns_ips()
    elif platform.system() == "Linux":
        dns_ips = get_unix_dns_ips()
    else:
        raise DnsError(f"unsupported platform: {platform.system()}")
    return dns_ips[0]  # return 1st dns entry


if __name__ == "__main__":
    main()
""" Tested with default macbookpro settings:

wifi 2g HughesNet ['192.168.0.1', '192.168.0.1']
wifi 5g HughesNet ['192.168.0.1', '192.168.0.1']

"""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" gpt2_sample - a little something to play with ...
    reference: https://towardsdatascience.com/roadmap-to-natural-language-processing-nlp-38a81dcff3a6
    ---
    Part of the [AutoSys][1] package

    Copyright (c) 2018 [Michael Treanor][2]

    AutoSys is licensed under the [MIT License][3]

    [1]: https://www.github.com/skeptycal/autosys
    [2]: https://www.twitter.com/skeptycal
    [3]: https://opensource.org/licenses/MIT
    """

import tensorflow as tf
from transformers import TFGPT2LMHeadModel, GPT2Tokenizer

# Loading and setting up GPT2 Open AI Transformer Model
tokenizer = GPT2Tokenizer.from_pretrained("gpt2")
model = TFGPT2LMHeadModel.from_pretrained("gpt2",
                                          pad_token_id=tokenizer.eos_token_id)
# Encoding the starting point of the sentence we want to predict
input_data = tokenizer.encode("A book is a mirror that offers us only " +
                              "what we already carry inside us.",
                              return_tensors='tf')
# Generating Output String
output = model.generate(
    input_data,
    do_sample=True,
    max_length=50,
    top_k=30
)

print(tokenizer.decode(output[0], skip_special_tokens=True))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" INI files
    ---
    INI files - utilities for parsing ini files into usable datasets

    An INI file is basically a dictionary of keys and values with headings and comments mixed in.

    The most useful data format that maps INI file format is the Python dictionary, which is nearly identical to the JSON specification.

    In the data format used here, the headings are used as 'keys' in key-value pairs and the 'values' are a dictionary of the actual items. This format is also trivial to translate and export the data into JSON format.

    Example:

    ```
    This .editorconfig file is processed into the dictionary shown below:

    # EditorConfig is awesome: http://EditorConfig.org

    # top-most EditorConfig file
    root = true

    # Tab indentation
    [*]
    indent_style = space
    indent_size = 4
    trim_trailing_whitespace = true
    end_of_line = lf
    insert_final_newline = true

    # Matches multiple files with brace expansion notation
    # Set default charset
    [*.{js,py}]
    charset = utf-8

    # The indent size used in the `package.json` file cannot be changed
    # https://github.com/npm/npm/pull/3180#issuecomment-16336516
    [{.travis.yml,npm-shrinkwrap.json,package.json}]
    indent_style = space
    indent_size = 2
    ```

    Data after processing:

    {
        "top":{
            "root": True,
        },
        "*": {
            "indent_style": "space",
            "indent_size": "4",
            "trim_trailing_whitespace": "true",
            "end_of_line": "lf",
            "insert_final_newline": "true",
        },
        "*.{js,py}":{
            "charset": "utf-8",
        }
    }

        []


    (Comments can optionally be ignored or processed into labels. They are ignored in the above example.)



    AutoSys
    ---
    Part of the [AutoSys][1] package

    Copyright (c) 2018 [Michael Treanor][2]

    AutoSys is licensed under the [MIT License][3]

    [1]: https://www.github.com/skeptycal/autosys
    [2]: https://www.twitter.com/skeptycal
    [3]: https://opensource.org/licenses/MIT
    """
# 'Standard Library'
import io
import re

from dataclasses import Field, dataclass, field
from pathlib import Path

# 'package imports'
from autosys.log import logging
from autosys.parse import ini_files

from typing import List


''' #TODO - features to add to TextFile
    _access_log: List = field(init=False)
    _track: bool = False
    readlines: bool = True
    '''


class IniFileError(Exception):
    ''' Ini file error. '''


@dataclass
class TextFile:
    file_name: str
    comment_char: str = '#;'
    keep_comments: bool = True
    _raw_data: str = ''
    _path: Path = field(init=False)

    def __post_init__(self):
        self._path = self._get_path(self.file_name)
        self._access_log = []
        self._raw_data = self._get_data()
        if not self.keep_comments:
            self.data = self._drop_comments()

    @staticmethod
    def resolve(path_name: (str, Path)) -> (Path):
        ''' Return a Path that has been resolved and verified. '''
        if Path(path_name).resolve().is_file():
            return Path(path_name).resolve()
        raise IOError(f'The file {path_name} was not found.')

    def _get_path(self, path_name) -> (Path):
        tmp: Path = None
        try:  # try script directory
            return self.resolve(path_name)
        except:
            try:  # try PWD
                return self.resolve(Path().cwd() / Path(path_name).name)
            except:
                raise

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, path_name):
        if not self._path:
            self._path = self._get_path(path_name)
        return self._path

    def _get_data(self):
        try:
            with open(self.path, mode='rt') as fd:
                lines = fd.readlines()
        except:
            raise IniFileError(f"Error reading data from file <{self.path}>.")

    def _drop_line_comment(self, line):
        retval = line.split(self.comment_char)[0].strip()
        if retval:
            return retval
        return None

    def _drop_comments(self):
        ''' strip comments and whitespace. '''
        tmp: List = []
        for line in self._raw_data:
            line = self._drop_line_comment(line)
            if line:
                tmp.append(line)
        return tmp

    def as_list(self):
        return [
            f"{k:<20.20}: {v}" for k, v in self.__dict__.items()
            if not k.startswith('_')
        ]

    def pretty(self):
        return '\n'.join(self.as_list())


@dataclass
class IniFile(TextFile):
    heading_pattern: re.Pattern = r'\[.*\]'

    def __post_init__(self):
        super().__post_init__()
        self.readlines = True
        self.find_headings()

    def find_headings(self):
        tmp: List = []
        for line in self._raw_data:
            if re.search(self.heading_pattern, line):
                tmp.append(line.lstrip('[').rstrip(']').strip())
                print(line.lstrip('[').rstrip(']').strip())


s = IniFile(file_name='sample_editorconfig.ini')

print(s.pretty())

# print(sample_ini)

#? -----------------------------------------
#? PyINI
#? https://github.com/PolyEdge/PyINI


class ini:
    class _ini_group:
        def __init__(self, name):
            "Read ini from file object or string given -> ini object"
            self.name = name
            self._items = []

        def __setitem__(self, item, val):
            if type(item) != str:
                raise TypeError("Invalid subscript type")
            for x in range(0, self._items.__len__()):
                if self._items[x][0] == item:
                    self._items[x][1] = val
                    return
            self._items.append([item, val])

        def __getitem__(self, item):
            if type(item) != str:
                raise TypeError("Invalid subscript type")
            for x in range(0, self._items.__len__()):
                if self._items[x][0] == item:
                    return self._items[x][1]
            raise IndexError("Key '" + item + "' not found.")

        def __delitem__(self, item):
            if type(item) != str:
                raise TypeError("Invalid subscript type")
            for x in range(0, self._items.__len__()):
                if self._items[x][0] == item:
                    del self._items[x]
                    return
            raise IndexError("Key '" + item + "' not found.")

        def __iter__(self):
            return iter({x[0]: x[1] for x in self._items if x[0] != -1})

        def __repr__(self):
            return '<ini group {' + (', '.join([
                '"' + str(x[0]) + '": "' + str(x[1]) + '"'
                for x in self._items if x[0] != -1
            ])) + '}>'

        def _add_blank_line(self):
            self._items.append([-1, "blankline"])

        def _add_comment(self, comment):
            self._items.append([-1, "comment", comment])

        def _add_raw(self, raw):
            self._items.append([-1, "raw", raw])

    def __init__(self, ini):
        if type(ini) == io.TextIOWrapper:
            if ini.mode == 'r':
                data = ini.read()
                ini.close()
            else:
                raise ValueError(
                    'File object does not support reading ("r") mode')
        else:
            if type(ini) == str:
                data = ini
            else:
                raise TypeError(
                    'Expected "str" or "io.TextIOWrapper" but got ' +
                    (str(ini.__class__).split()[1][:-1]))
        self._groups = [self._ini_group("__default__")]
        for x in [y.strip() for y in str(data).rsplit(sep='\n')]:
            if x == '':
                self._groups[-1]._add_blank_line()
            elif x[0] == '#':
                self._groups[-1]._add_comment(x[1:])
            elif x[0] == '[':
                self._groups.append(self._ini_group(x[1:-1]))
            elif len(x.split('=', 1)) > 1:
                self._groups[-1][x.split('=', 1)[0].strip()] = x.split(
                    '=', 1)[1].strip()
            else:
                self._groups[-1]._add_raw(x)
        self.create_group = self._create_group
        self.dump = self._dump

    def __setitem__(self, item, val):
        raise TypeError("cannot assign to base ini group")

    def __getitem__(self, item):
        if type(item) != str:
            raise TypeError("Invalid subscript type")
        for x in range(0, self._groups.__len__()):
            if self._groups[x].name == item:
                return self._groups[x]
        raise IndexError("Key '" + item + "' not found.")

    def __delitem__(self, item):
        if type(item) != str:
            raise TypeError("Invalid subscript type")
        for x in range(0, self._groups.__len__()):
            if self._groups[x].name == item:
                del self._groups[x]
                return
        raise TypeError("cannot delete nonexisting ini group.")

    def __iter__(self):
        return iter({x.name: x for x in self._groups})

    def __repr__(self):
        return '<ini group collection [' + (', '.join(
            ['"' + x.name + '"' for x in self._groups])) + ']>'

    def _create_group(self, name):
        for x in range(0, self._groups.__len__()):
            if self._groups[x].name == item:
                raise TypeError("cannot create existing ini group.")
        self._groups.append(self._ini_group(name))

    def _dump(self, stream=None):
        output = []
        for x in self._groups:
            if x.name != '__default__':
                output.append('[' + x.name + ']')
            for y in x._items:
                if (y[0] == -1) and (y[1] == "blankline"):
                    output.append("")
                elif (y[0] == -1) and (y[1] == "comment"):
                    output.append("#" + y[2])
                elif (y[0] == -1) and (y[1] == "raw"):
                    output.append(y[2])
                else:
                    output.append(str(y[0]) + ' = ' + str(y[1]))
        output = '\n'.join(output)
        if stream == None:
            return output
        elif (stream.mode in ['w', 'w+']) and (not stream.closed):
            stream.write(output)
            stream.close()
        elif stream.closed:
            raise IOError('Cannot write to closed stream')
        else:
            raise ValueError('Invalid file mode')


''' sub-class that is inside of class ini '''


class _ini_group:
    def __init__(self, name):
        "Read ini from file object or string given -> ini object"
        self.name = name
        self._items = []

    def __setitem__(self, item, val):
        if type(item) != str:
            raise TypeError("Invalid subscript type")
        for x in range(0, self._items.__len__()):
            if self._items[x][0] == item:
                self._items[x][1] = val
                return
        self._items.append([item, val])

    def __getitem__(self, item):
        if type(item) != str:
            raise TypeError("Invalid subscript type")
        for x in range(0, self._items.__len__()):
            if self._items[x][0] == item:
                return self._items[x][1]
        raise IndexError("Key '" + item + "' not found.")

    def __delitem__(self, item):
        if type(item) != str:
            raise TypeError("Invalid subscript type")
        for x in range(0, self._items.__len__()):
            if self._items[x][0] == item:
                del self._items[x]
                return
        raise IndexError("Key '" + item + "' not found.")

    def __iter__(self):
        return iter({x[0]: x[1] for x in self._items if x[0] != -1})

    def __repr__(self):
        return '<ini group {' + (', '.join([
            '"' + str(x[0]) + '": "' + str(x[1]) + '"'
            for x in self._items if x[0] != -1
        ])) + '}>'

    def _add_blank_line(self):
        self._items.append([-1, "blankline"])

    def _add_comment(self, comment):
        self._items.append([-1, "comment", comment])

    def _add_raw(self, raw):
        self._items.append([-1, "raw", raw])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" Quick Colors

    Make adding color to terminal output as simple as possible. Use quick, short tags in place of Colorama variables. Resembles basic html tags.
    Includes a simple parser to do the tag replacement and automatic string processing.

    ```
    p = QuickColor('')

    p.rint('<r>red <b>blue')

    r = '<R>'
    y = '<y>'

    p.rint(f'{r}red {y}yellow')

    p = QuickColor(f'{r}red {y}yellow')

    print(p)

    color_list = ['<c>', 'cyan ', '<g>', 'green']
    p.rint(color_list)

    ```

    The 'quick print' codes are inline string literals that are similar to html tags and are used as placeholders for automatic replacement with ansi color strings similar to those used by the Pypi package Colorama.


    ```
    # standard colors
    <K>  or  <k>    # K is black
    <B>  or  <b>    # Blue
    <C>  or  <c>    # Cyan
    <G>  or  <g>    # Green
    <M>  or  <m>    # Magenta
    <R>  or  <r>    # Red
    <W>  or  <w>    # White
    <Y>  or  <y>    # Yellow

    # light colors
    <LK> or <lk>    # light black
    <LB> or <lb>    # light Blue
    <LC> or <lc>    # light Cyan
    <LG> or <lg>    # light Green
    <LM> or <lm>    # light Magenta
    <LR> or <lr>    # light Red
    <LW> or <lw>    # light White
    <LY> or <ly>    # light Yellow

    <x> or <X>    # X is for reset
    ```

    They may be upper or lower case. If <reset_color> is set to true (the default), a trailing 'reset' code is automatically added to eliminate color bleeding to upcoming lines.

    Common available methods and attributes:
    ```
    # instantiate
    p = QuickColor('')

    # parse the current string (or new string <s>) and return it
    p.parse(s='optional sequence')

    # print the current string (or new string <s>)
    p.rint(s='optional sequence')

    # visual test of current terminal configuration
    p.test(s='optional values')

    ```


    More info about [Colorama][4]
    ---
    Part of the [AutoSys][1] package

    Copyright (c) 2018 [Michael Treanor][2]

    AutoSys is licensed under the [MIT License][3]

    [1]: https://www.github.com/skeptycal/autosys
    [2]: https://www.twitter.com/skeptycal
    [3]: https://opensource.org/licenses/MIT
    [4]: https://pypi.org/project/colorama/
    """

from dataclasses import Field, dataclass, field
from io import TextIOWrapper
from os import linesep as NL
from sys import stdout

from typing import Dict, Final, List, Sequence


@dataclass
class QuickColor:
    ''' Quick Colors

        Make adding color to terminal output as simple as possible. Use quick, short tags in place of Colorama variables. Resembles basic html tags.
        Includes a simple parser to do the tag replacement and automatic string processing.

        ```
        p = QuickColor('')

        p.rint('<r>red <b>blue')

        r = '<R>'
        y = '<y>'

        p.rint(f'{r}red {y}yellow')

        p = QuickColor(f'{r}red {y}yellow')

        print(p)

        color_list = ['<c>', 'cyan ', '<g>', 'green']
        p.rint(color_list)

        ```

        The 'quick print' codes are inline string literals that are similar to html tags and are used as placeholders for automatic replacement with ansi color strings similar to those used by the Pypi package Colorama.


        ```
        # standard colors
        <K>  or  <k>    # K is black
        <B>  or  <b>    # Blue
        <C>  or  <c>    # Cyan
        <G>  or  <g>    # Green
        <M>  or  <m>    # Magenta
        <R>  or  <r>    # Red
        <W>  or  <w>    # White
        <Y>  or  <y>    # Yellow

        # light colors
        <LK> or <lk>    # light black
        <LB> or <lb>    # light Blue
        <LC> or <lc>    # light Cyan
        <LG> or <lg>    # light Green
        <LM> or <lm>    # light Magenta
        <LR> or <lr>    # light Red
        <LW> or <lw>    # light White
        <LY> or <ly>    # light Yellow

        <x> or <X>    # X is for reset
        ```

        They may be upper or lower case. If <reset_color> is set to true (the default), a trailing 'reset' code is automatically added to eliminate color bleeding to upcoming lines.

        Common available methods and attributes:
        ```
        # instantiate
        p = QuickColor('')

        # parse the current string (or new string <s>) and return it
        p.parse(s='optional sequence')

        # print the current string (or new string <s>)
        p.rint(s='optional sequence')

        # visual test of current terminal configuration
        p.test(s='optional values')

        ```

        '''
    args: Sequence[str]  # Initial arguments passed in as any Sequence type

    reset_color: Final[bool] = True  # True => reset colors at end of line
    RESET_ALL: Final[str] = '\x1b[0m'  # Code to use for reset

    # Print function default keyword arguments
    sep: Final[str] = ''  # not implemented
    end: Final[str] = NL
    file: Final[TextIOWrapper] = stdout
    flush: Final[bool] = False

    # container for current parsed string
    _string: str = ''
    # container for current prepared endline reset string
    _reset: str = ''
    # dictionary of color codes for parsing
    _quick_colors_dict: Dict[str, str] = field(init=False, hash=True)
    # regex pattern for locating tags (not currently implemented)
    # _RE_QUICK_COLOR_TAG: Final[re.Pattern] = re.compile(r'<\w{1,2}>')

    def __post_init__(self):
        ''' Finish configuration of the class instance by filling the
            color dictionary and parsing the initial <args> argument.
            '''
        self._quick_colors_dict = self._get_dict()
        self._string = self.parse(self.args)
        self._reset = self.RESET_ALL * self.reset_color

    def __str__(self):
        ''' Return the a parsed string representing the current value
            of the instance string storage variable.

            To return a parsed string with a new value, use the 'parse'
            method and pass in a new string <s>.'''
        return self.parse()

    def _get_dict(self):
        return {
            '<K>': '\x1b[30m',  # K is black
            '<B>': '\x1b[34m',  # Blue
            '<C>': '\x1b[36m',  # Cyan
            '<G>': '\x1b[32m',  # Green
            '<M>': '\x1b[35m',  # Magenta
            '<R>': '\x1b[31m',  # Red
            '<W>': '\x1b[37m',  # White
            '<Y>': '\x1b[33m',  # Yellow
            '<LK>': '\x1b[90m',  # light colors
            '<LB>': '\x1b[94m',
            '<LC>': '\x1b[96m',
            '<LG>': '\x1b[92m',
            '<LM>': '\x1b[95m',
            '<LR>': '\x1b[91m',
            '<LW>': '\x1b[97m',
            '<LY>': '\x1b[93m',
            '<X>': '\x1b[39m',  # X is for reset
        }

    def parse(self, s: Sequence = ''):
        ''' Return a colorized string ready to print or use. '''
        if s:
            if isinstance(s, str):
                self._string = s
            else:
                self._string = ''.join(s)
        else:
            if not self._string:
                self._string = ''.join(self.args)
        for k, v in self._quick_colors_dict.items():
            self._string = self._string.replace(k, v)
            k = k.lower()
            self._string = self._string.replace(k, v)
        return f"{self._string}{self._reset}"

    def rint(self, s: Sequence = ''):
        ''' Print a parsed string using the QuickColor options
            <end>, <file>, <flush> as print keyword arguments.

            A new string can optionally be passed in as <s> so a single
            class instance can be reused, saving resources.
            '''
        print(self.parse(s=s), end=self.end,
              file=self.file, flush=self.flush)

    def test(self, s: Sequence = ''):
        ''' Assorted visual tests. Use <s> to send in more. '''
        print('QuickColor Tests: ')
        if s:
            self.rint(s=s)
        red = '<r>'
        blue = '<b>'
        color_test = f"<r>red <b>blue"
        print('First test string: ', color_test, ' => ', p.parse(color_test))
        self.rint('<m>magenta <lg>light green')
        r = '<R>'
        y = '<y>'
        self.rint(f'{r}red {y}yellow')
        color_list = ['<c>', 'cyan ', '<g>', 'green']
        self.rint(color_list)


p = QuickColor('')
# p.test()

''' References ...

    Colorama Color Constants are:
        - Fore: BLACK, RED, GREEN, YELLOW, BLUE,
        MAGENTA, CYAN, WHITE, RESET.
        - Back: BLACK, RED, GREEN, YELLOW, BLUE,
        MAGENTA, CYAN, WHITE, RESET.
        - Style: DIM, NORMAL, BRIGHT, RESET_ALL

    colorama_Fore: Dict[str, str] = {
        'BLACK': '\x1b[30m',
        'BLUE': '\x1b[34m',
        'CYAN': '\x1b[36m',
        'GREEN': '\x1b[32m',
        'LIGHTBLACK_EX': '\x1b[90m',
        'LIGHTBLUE_EX': '\x1b[94m',
        'LIGHTCYAN_EX': '\x1b[96m',
        'LIGHTGREEN_EX': '\x1b[92m',
        'LIGHTMAGENTA_EX': '\x1b[95m',
        'LIGHTRED_EX': '\x1b[91m',
        'LIGHTWHITE_EX': '\x1b[97m',
        'LIGHTYELLOW_EX': '\x1b[93m',
        'MAGENTA': '\x1b[35m',
        'RED': '\x1b[31m',
        'RESET': '\x1b[39m',
        'WHITE': '\x1b[37m',
        'YELLOW': '\x1b[33m',
    }
    colorama_Back: Dict[str, str] = {
        'BLACK': '\x1b[40m',
        'BLUE': '\x1b[44m',
        'CYAN': '\x1b[46m',
        'GREEN': '\x1b[42m',
        'LIGHTBLACK_EX': '\x1b[100m',
        'LIGHTBLUE_EX': '\x1b[104m',
        'LIGHTCYAN_EX': '\x1b[106m',
        'LIGHTGREEN_EX': '\x1b[102m',
        'LIGHTMAGENTA_EX': '\x1b[105m',
        'LIGHTRED_EX': '\x1b[101m',
        'LIGHTWHITE_EX': '\x1b[107m',
        'LIGHTYELLOW_EX': '\x1b[103m',
        'MAGENTA': '\x1b[45m',
        'RED': '\x1b[41m',
        'RESET': '\x1b[49m',
        'WHITE': '\x1b[47m',
        'YELLOW': '\x1b[43m'
    }
    colorama_Style: Dict[str, str] = {
        'BRIGHT': '\x1b[1m',
        'DIM': '\x1b[2m',
        'NORMAL': '\x1b[22m',
        'RESET_ALL': '\x1b[0m'
    }
    '''



# 'Standard Library'
from dataclasses import dataclass


@dataclass
class AnsiCodes:
    """ ANSI color magic 🦄  """

    # - some favorites
    MAIN: str = "\x1B[38;5;229m"
    WARN: str = "\x1B[38;5;203m"
    BLUE: str = "\x1B[38;5;38m"
    GO: str = "\x1B[38;5;28m"
    CHERRY: str = "\x1B[38;5;124m"
    CANARY: str = "\x1B[38;5;226m"
    ATTN: str = "\x1B[38;5;178m"
    RAIN: str = "\x1B[38;5;93m"
    WHITE: str = "\x1B[37m"
    RESET: str = "\x1B[0m"  # - reset ansi effects
    RESTORE: str = "\x1B[0m"  # - alias of RESET

    # - Encode ANSI 7 bit effect set
    # BOLD: str = "\x1B[1m"
    B: str = "\x1B[1m"
    FAINT: str = "\x1B[2m"
    ITALIC: str = "\x1B[3m"
    IT: str = "\x1B[3m"  # alias of ITALIC
    UNDERLINE: str = "\x1B[4m"
    UL: str = "\x1B[4m"  # alias of UNDERLINE
    BLINK: str = "\x1B[5m"
    REVERSE: str = "\x1B[7m"
    CONCEAL: str = "\x1B[8m"
    STRIKE: str = "\x1B[9m"
    FRAME: str = "\x1B[51m"
    CIRCLE: str = "\x1B[52m"
    OVERLINE: str = "\x1B[53m"

    # - Encode ANSI 7 bit foreground color set
    BLACK: str = "\x1B[30m"
    RED: str = "\x1B[31m"
    GREEN: str = "\x1B[32m"
    YELLOW: str = "\x1B[33m"
    BLUE7: str = "\x1B[34m"  # defined above
    MAGENTA: str = "\x1B[35m"
    CYAN: str = "\x1B[36m"
    # WHITE: str = "\x1B[37m"  # defined above

    # - Encode extended ANSI 7 bit foreground color set
    BRIGHTBLACK: str = "\x1B[30;1m"
    BRIGHTRED: str = "\x1B[31;1m"
    BRIGHTGREEN: str = "\x1B[32;1m"
    BRIGHTYELLOW: str = "\x1B[33;1m"
    BRIGHTBLUE: str = "\x1B[34;1m"
    BRIGHTMAGENTA: str = "\x1B[35;1m"
    BRIGHTCYAN: str = "\x1B[36;1m"
    BRIGHTWHITE: str = "\x1B[37;1m"

    # - Encode ANSI 7 bit background color set
    BBLACK: str = "\x1B[40m"
    BRED: str = "\x1B[41m"
    BGREEN: str = "\x1B[42m"
    BYELLOW: str = "\x1B[43m"
    BBLUE: str = "\x1B[44m"
    BMAGENTA: str = "\x1B[45m"
    BCYAN: str = "\x1B[46m"
    BWHITE: str = "\x1B[47m"

    # - Encode extended ANSI 7 bit background color set
    BBRIGHTBLACK: str = "\x1B[40;1m"
    BBRIGHTRED: str = "\x1B[41;1m"
    BBRIGHTGREEN: str = "\x1B[42;1m"
    BBRIGHTYELLOW: str = "\x1B[43;1m"
    BBRIGHTBLUE: str = "\x1B[44;1m"
    BBRIGHTMAGENTA: str = "\x1B[45;1m"
    BBRIGHTCYAN: str = "\x1B[46;1m"
    BBRIGHTWHITE: str = "\x1B[47;1m"



from collections import OrderedDict


def _attr_key(attr):
    """Returns appropriate key for sorting attribute names

    Attribute names are a tuple of ``(namespace, name)`` where namespace can be
    ``None`` or a string. These can't be compared in Python 3, so we convert
    the ``None`` to an empty string.

    """
    key = (attr[0][0] or ''), attr[0][1]
    return key


def alphabetize_attributes(attrs):
    """Takes a dict of attributes (or None) and returns them alphabetized"""
    if not attrs:
        return attrs

    return OrderedDict([(k, v)
                        for k, v in sorted(attrs.items(), key=_attr_key)])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

""" AutoSys.py

Usage:
  autosys.py [run|add|del|list|info] <script_name> [-q | -v]
  autosys.py [start|stop|config|test] <service_name> [--args=<args>]
  autosys.py info [--args=<args>]
  autosys.py (-h | --help)
  autosys.py --version

Options:
  -h --help     Show this screen.
  --version     Show version.
  --args=<args>  Arguments to pass along.

 `AutoSys` package
        copyright (c) 2018 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`

"""

from docopt import docopt

if __name__ == "__main__":
    test_args: List[str] = [ "--verbose", "--help" ]
    arguments = docopt(__doc__, version=f"AutoSys version{}")
    print(arguments)
docopt(doc, argv=None, help=True, version=None, options_first=False)



# -*- coding: utf-8 -*-
# 'Standard Library'
import time

from time import perf_counter_ns as timer

# 'third party'
import numpy as np

size_of_vec = 1000


def pure_python_version():
    t1 = timer()
    X = range(size_of_vec)
    Y = range(size_of_vec)
    Z = [X[i] + Y[i] for i in range(len(X))]
    return timer() - t1


def numpy_version():
    t1 = timer()
    X = np.arange(size_of_vec)
    Y = np.arange(size_of_vec)
    print(X.shape)
    print(Y.shape)
    Z = X + Y
    return timer() - t1


t1 = pure_python_version()
t2 = numpy_version()
print(f"Time 1: {t1:8}")
print(f"Time 2: {t2:8}")
print()
faster = t1
slower = t2
if t1 > t2:
    faster, slower = slower, faster
print(f"{faster} is {t1 / t2:7.5} % faster than {slower}!")



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

# 'package imports'
import tweepy

# AUTHenticate to Twitter
AUTH = tweepy.OAUTHHandler("CONSUMER_KEY", "CONSUMER_SECRET")
AUTH.set_access_token("ACCESS_TOKEN", "ACCESS_TOKEN_SECRET")

# Create API object
api = tweepy.API(AUTH)

# Create a tweet
api.update_status("Hello Tweepy")



#!/usr/bin/env python

# Copyright 2007-2016 The Python-Twitter Developers

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# ----------------------------------------------------------------------

# This file demonstrates how to track mentions of a specific set of users in
# english language and archive those mentions to a local file. The output
# file will contain one JSON string per line per Tweet.

# To use this example, replace the W/X/Y/Zs with your keys obtained from
# Twitter, or uncomment the lines for getting an environment variable. If you
# are using a virtualenv on Linux, you can set environment variables in the
# ~/VIRTUALENVDIR/bin/activate script.

# If you need assistance with obtaining keys from Twitter, see the instructions
# in doc/getting_started.rst.

# 'Standard Library'
import json
import os

# 'package imports'
from twitter import Api

# Either specify a set of keys here or use os.getenv('CONSUMER_KEY') style
# assignment:

CONSUMER_KEY = "WWWWWWWW"
# CONSUMER_KEY = os.getenv("CONSUMER_KEY", None)
CONSUMER_SECRET = "XXXXXXXX"
# CONSUMER_SECRET = os.getenv("CONSUMER_SECRET", None)
ACCESS_TOKEN = "YYYYYYYY"
# ACCESS_TOKEN = os.getenv("ACCESS_TOKEN", None)
ACCESS_TOKEN_SECRET = "ZZZZZZZZ"
# ACCESS_TOKEN_SECRET = os.getenv("ACCESS_TOKEN_SECRET", None)

# Users to watch for should be a list. This will be joined by Twitter and the
# data returned will be for any tweet mentioning:
# @twitter *OR* @twitterapi *OR* @support.
USERS = ["@twitter", "@twitterapi", "@support"]

# Languages to filter tweets by is a list. This will be joined by Twitter
# to return data mentioning tweets only in the english language.
LANGUAGES = ["en"]

# Since we're going to be using a streaming endpoint, there is no need to worry
# about rate limits.
api = Api(CONSUMER_KEY, CONSUMER_SECRET, ACCESS_TOKEN, ACCESS_TOKEN_SECRET)


def main():
    with open("output.txt", "a") as f:
        # api.GetStreamFilter will return a generator that yields one status
        # message (i.e., Tweet) at a time as a JSON dictionary.
        for line in api.GetStreamFilter(track=USERS, languages=LANGUAGES):
            f.write(json.dumps(line))
            f.write("\n")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" as_pathlib.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal

if True:
    import os
    import pathlib
    import sys
    import time
    from dataclasses import dataclass, field
    from pathlib import WindowsPath, PosixPath, PurePath, Path
    from typing import Any, Dict, FrozenSet, List, Sequence, Tuple


def get_env_path() -> str:
    """ Return system path """
    return os.getenv("PATH")


class PPath(pathlib.PurePath):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def __init__(self, path_name: pathlib.PurePath, args):
        if os.name == "nt":
            Path = WindowsPath
        else:
            Path = PosixPath
        self.PP = path_name
        if SET_DEBUG:
            print(self.PP.as_uri())

    def subs(self):
        return [x for x in PP.iterdir() if x.is_dir()]

    def exists(self):
        return self.PP.exists()

    def is_dir(self):
        return self.PP.is_dir()

    def next_line(self):
        try:
            with self.PP.open() as f:
                return f.readline()
        except OSError as e:
            return e

    def name(self) -> str:
        return self.PP.parts()

    def str(self):
        return str(self.PP)

    def ls(self, args):
        for x in PP.iterdir():
            print(x)


def _get_builtins():
    return sys.builtin_module_names


def py_path() -> List[str]:
    """ Return list of current python path elements. """
    try:
        return os.environ["PYTHONPATH"].split(os.pathsep)
    except KeyError:
        return []


def py3up() -> bool:
    """ Return True if 'Python >= 3' else False

        If you want to detect pre-Python 3 and don't want to import anything...
        ... you can (ab)use list comprehension scoping changes
    """
    # https://stackoverflow.com/questions/1093322/how-do-i-check-what-version-of-python-is-running-my-script/35294211
    # https://stackoverflow.com/a/52825819/9878098
    return (lambda x: [x for x in [False]] and None or x)(True)


def pyver() -> str:
    """ Returns string with python version number in major.minor.micro format.
            (e.g. 3.7.3  or  2.7.12)
    """
    try:
        return ".".join(str(i) for i in __import__("sys").version_info[:3])
    except OSError:
        return ""


def py_shell() -> str:
    """ Returns string containing current python shell name. """

    shell: str = "python"
    PY_ENV = os.environ
    PY_BASE = os.path.basename(PY_ENV["_"])
    if "JPY_PARENT_PID" in PY_ENV:
        shell = "ipython notebook"
    elif "pypy" in PY_ENV:
        shell = "pypy"
    elif "jupyter-notebook" in PY_BASE:
        shell = "jupyter notebook"
    elif "ipython" in PY_BASE:
        shell = "ipython"
    else:
        try:
            import platform
        except ImportError:
            pass
        else:
            shell = platform.python_implementation()
    # print("pyshell() output: ", shell.strip())
    return shell.strip()


def _pprint_globals():
    """
    Pretty Print all global variables.
    Designed for debugging purposes.
    """
    print()
    print("Globals: ")
    print("*" * 40)
    width = max(len(i) for i in globals())
    print(f"(key width: {width})")
    for s in globals():
        print("{:<15.15} : {:<64.64}".format(s, str(globals().get(s))))
    print()


def iterable(obj):
    return hasattr(obj, "__iter__") or hasattr(obj, "__getitem__")


# def njoin(l: List[str]) -> str:
#     return '\n'.join(l)


@timeit
def time_iter_check1(n: int = 10000, var: object = None):
    if not var:
        var = "*" * 2000
    for _ in range(n):
        result = iterable(var)
    return result


# time_iter_check2 is xx% faster than time_iter_check1


@timeit
def time_iter_check2(
    n: int = 10000, var: object = None, i: int = 10
) -> List[float]:
    if not var:
        var = "*" * 2000

    for _ in range(n):
        try:
            result = iter(var)
            result = True
        except TypeError:
            result = False


def time_iter_check(
    n: int = 10000, var: object = None, i: int = 10
) -> List[float]:
    results = []
    for _ in range(i):
        results.append(time_iter_check1(n, var))

        time_iter_check2(n, var)


def njoin(s, delimeter: str = ",") -> str:
    """
    Return a string of lines (with LF) from a delimited string or iterable object.
    """
    if isinstance(s, str):
        return "\n".join(s.split(delimeter))
    try:
        _ = iter(s)
        return "\n".join(_)
    except TypeError:
        pass


def _pprint_code_tests(tests: List[Exception]):
    pass


def _get_functions():
    pass


def _pprint_dict_table(
    data: Dict[Any, Any] = globals(), title: str = "", borders: str = "text"
):
    """
    Pretty Print dictionary in table format.
    parameters:
        data: Dict[Any, Any] - dictionary containing table data
        title: str - Displayed Title of Data Table (Title Case Used)
        borders: str - border format (single, double, graphic, text*)
    """
    if title == "":
        title = data.__repr__()
    print(data.__repr__())
    print(data.__str__())
    maxwidth: int = 75  # 80 - 1 for each outer border and 3 for center padding
    width: int = max(len(i) for i in data) + 1
    width2: int = maxwidth - width
    if SET_DEBUG:
        print()
        print("Title: ", title.title())
        print("*" * len(title))
        print("width: ", width)
        print("width2: ", width2)

    fmt = (
        "{:<"
        + str(width)
        + "."
        + str(width)
        + "} : "
        + "{:<"
        + str(width2)
        + "."
        + str(width2)
        + "}"
    )

    print("*" * 80)
    for s in globals():
        print(fmt.format(s, str(globals().get(s))))
    print("*" * 80)


def _execute_test_code(tests: List[str]) -> List[Exception]:
    result: List[Exception] = []
    for test in tests:
        try:
            # run test code
            print("=> ", test)
            exec(test)
        except Exception as e:
            result.append(e)
    return result


def _run_tests(tests: List[str] = []) -> str:
    if tests == []:
        tests: List[str] = [
            "_pprint_globals()",
            'print("basename: ", basename(__file__))',
            "print(1/0)",
        ]

    results = _execute_test_code(tests)
    for e in results:
        print("e.__class__.__name__", e.__class__.__name__)
        # log.exception(e)
        print("e: ", e)
        print("e.args: ", e.args)
        print("type(e): ", type(e))


if __name__ == "__main__":
    # log = logging.getLogger()
    print(_run_tests())
    print()
    print("... Tests Complete.")
    _pprint_dict_table

    print(njoin(_get_builtins()))
    print()
    print(njoin(get_env_path(), ":"))
    print(iterable("this"))
    print(iterable(5343))
    print(iterable([1, 2, 3, 4]))
    print(iterable(["a", "b"]))
    s1 = set(x for x in "this")
    print(iterable(s1))
    print(iterable(pathlib.Path()))
    print(time_iter_check(1000000))
    print(basename(__doc__))



#!/usr/bin/env python
# grep_some_condition.py
# https://stackoverflow.com/a/5463419

# 'package imports'
from remove_lines import remove_line

line = line.replace("test", "testZ")
line = line.replace("test", "testZ")
line = line.replace("test", "testZ")
line = line.replace("test", "testZ")
line = line.replace("test", "testZ")
line = line.replace("test", "testZ")
line = line.replace("test", "testZ")

remove_line(example_file.py)

# T test

# The lines below (#TT ... ) should be removed
# TT test
# TT test2
# TT test3
# TT test asdfasdf



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
def check_encoding(file_name: str = __file__,
                   default: str = "utf-8",
                   verbose: bool = False):
    """ Return encoding specified at the top of file_name.
        If none is specified, return default.

        In other words, find and return this thing:
        ```
        # -*- coding: utf-8 -*-
        ```

        see https://www.python.org/dev/peps/pep-0263/"""
    import re

    pattern = re.compile(rb"[en]?coding[:=\s]+([-\w]*)")

    with open(file_name, mode='rb') as fp:
        lines: bytes = fp.read(300)
    retval = re.findall(pattern, lines)[0].decode("ascii") or default

    if verbose:
        print(f'-- Checking file {file_name}')
        print('-- First few lines')
        print(lines)
        print(f'-- returning encoding match: {retval}')

    return retval


check_encoding(verbose=True)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" Part of the `AutoSys` package - utilities for macOS apps
        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """

from typing import Dict, List, Sequence, Tuple

try:
    log.info("Loaded list_utils module.")
except:
    from autosys.log.autosys_logger import log


class StringError(ValueError):
    """ A string processing error has occurred. """


def dict2str(ignore_errors=True, sort_items=True, **kwargs):
    """ Return string from an iterable of variously typed arguments.
        Each item is converted separately, allowing errors in individual
        items to be ignored.

        data: iterable - iterable containing items
        arg_sep: str   - separator between list items

        ignore_errors  - Errors in types and formatting are ignored.
        """
    tmp: List[str] = []
    for k, v in kwargs.items():
        try:
            if isinstance(v, dict):
                v = dict_str(ignore_errors=ignore_errors, sort_items=sort_items, **v)
            tmp.append(f"{k}: {v}")
        except:
            pass
    if sort_items:
        tmp = sorted(tmp)


def arg2str(*data, arg_sep: str = "", ignore_errors=True, **kwargs):
    """ Return string from an iterable of variously typed arguments.
        Each item is converted separately, allowing errors in individual
        items to be ignored.

        data: iterable - iterable containing items
        arg_sep: str   - separator between list items

        ignore_errors  - Errors in types and formatting are ignored.
        (if errors are not ignored, a TypeError is raised.)
        """
    tmp: List[str] = []
    for arg in data:
        try:
            tmp.append(str(arg))
        except Exception as e:
            if not ignore_errors:
                log.error(e)
                raise StringError(e)

    if len(data) == 0 or data == None:
        return ""


def replace_all(
    needle: Sequence, haystack: Sequence, volunteer: Sequence = ""
) -> (Sequence):
    """ Return a sequence with all `needles` in `haystack` replaced with `volunteers` """
    return "".join(volunteer if c in needle else c for c in haystack)


def rep_whitelist(
    whitelist: Sequence, haystack: Sequence, volunteer: Sequence = "_"
) -> (Sequence):
    """ Return a sequence with all `needles` in `haystack` saved and all other characters replaced with `volunteers` """
    return "".join(volunteer if c not in whitelist else c for c in haystack)


def make_safe_id(haystack: Sequence, volunteer: Sequence = "_") -> (Sequence):
    """ Return a string that has only alphanumeric and _ characters.
        (This makes it legal for identifier names in python.)

        Illegal characters are replaced with `volunteer` (default `_`)
        """
    return "".join(volunteer if not c.isidentifier() else c for c in haystack)


if __name__ == "__main__":
    tmp: Dict = dict(globals())
    print(tmp)
#    for k, v in tmp.items():
#       try:
#           print(k, v)
#       except:
#           pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
''' find_dups_list.py - Find duplicates in a list or other sequence.
    Reference: https://www.techiedelight.com/find-duplicates-within-given-range-array/
    '''


def has_duplicates(A, k):
    ''' Returns True if A has duplicates within range k.

        Reference: https://www.techiedelight.com/find-duplicates-within-given-range-array/ '''
    # stores (element, index) pairs as (key, value) pairs
    _dict = {}

    # traverse the list
    for i, e in enumerate(A):

        # if the current element already exists in the dict
        if e in _dict:

            # return true if current element is repeated within range of k
            if i - _dict.get(e) <= k:
                print(_dict)
                return True

        # store elements along with their indices
        _dict[e] = i

    # we reach here when no element is repeated within range k
    return False


def has_duplicates_alt(A, k):
    ''' Returns True if A has duplicates within range k. '''
    # stores (element, index) pairs as (key, value) pairs
    _dict = {}

    # traverse the list
    for i, e in enumerate(A):

        # if the current element already exists in the dict
        if e in _dict:

            # return true if current element is repeated within range of k
            if i - _dict.get(e) <= k:
                print(_dict)
                return True

        # store elements along with their indices
        _dict[e] = i

    # we reach here when no element is repeated within range k
    return False


def example_has_duplicates():
    A = [5, 6, 8, 2, 4, 6, 9]
    k = 4

    if has_duplicates(A, k):
        print("Duplicates found")
    else:
        print("No Duplicates found")


if __name__ == '__main__':
    pass
    example_has_duplicates()



#!/Users/michaeltreanor/Documents/coding/python/autosys/autosys/pathpack/venv/bin/python3

import sys
import json
import argparse
from pprint import pformat

import jmespath
from jmespath import exceptions


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('expression')
    parser.add_argument('-f', '--filename',
                        help=('The filename containing the input data.  '
                              'If a filename is not given then data is '
                              'read from stdin.'))
    parser.add_argument('--ast', action='store_true',
                        help=('Pretty print the AST, do not search the data.'))
    args = parser.parse_args()
    expression = args.expression
    if args.ast:
        # Only print the AST
        expression = jmespath.compile(args.expression)
        sys.stdout.write(pformat(expression.parsed))
        sys.stdout.write('\n')
        return 0
    if args.filename:
        with open(args.filename, 'r') as f:
            data = json.load(f)
    else:
        data = sys.stdin.read()
        data = json.loads(data)
    try:
        sys.stdout.write(json.dumps(
            jmespath.search(expression, data), indent=4, ensure_ascii=False))
        sys.stdout.write('\n')
    except exceptions.ArityError as e:
        sys.stderr.write("invalid-arity: %s\n" % e)
        return 1
    except exceptions.JMESPathTypeError as e:
        sys.stderr.write("invalid-type: %s\n" % e)
        return 1
    except exceptions.UnknownFunctionError as e:
        sys.stderr.write("unknown-function: %s\n" % e)
        return 1
    except exceptions.ParseError as e:
        sys.stderr.write("syntax-error: %s\n" % e)
        return 1


if __name__ == '__main__':
    sys.exit(main())



# 'Standard Library'
import re

from os import linesep as NL

RE_DEFAULT_BITE_SIZE = 5
RE_DEFAULT_FLAGS = re.MULTILINE
RE_NUM = fr"\d{{1-{RE_DEFAULT_BITE_SIZE}}}*"
print(RE_NUM)


def brk(x):
    return re.findall(RE_NUM, x)


def breakup(s, bite_size=3, divider=" "):
    tmp = ""
    for i, c in enumerate(str(s)):
        tmp += c
        if (i + 1) % bite_size == 0:
            tmp += divider
    return tmp


def id_tail(x: object, n: int = 5) -> (str):
    return str(id(x))[-n:]



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-


newline_text: str = """ <aside>            NewLine, A Bit of Lore

    Newline is a control character or sequence of control characters in a
    character encoding specification (e.g. ASCII or EBCDIC) that is used to
    signify the end of a line of text and the start of a new one.

    In the mid-1800s, long before the advent of teletype machines, Morse code
    operators invented and used Morse code prosigns to encode white space
    text formatting in formal written text messages. Two 'A' characters sent
    without the normal spacing is used to indicate a new line of text.

    --------------------------------------------------------------------------

    - In computing and telecommunication, a control character or non-printing
        character (NPC) is a code point (a number) in a character set, that
        does not represent a written symbol.

    - During the 1960s, ISO and ASA (who later became ANSI), simultaneously
        developed ASCII for modern teleprinters. The sequence they agreed on
        for newline was CR+LF. This sequence carried forward to the first
        computers that had to share printers with other older devices.

    - The separation of newline into two functions concealed the fact that the
        print head could not return from the far right to the beginning of the
        next line in time to print the next character. Any character printed
        after a CR would often print as a smudge in the middle of the page
        while the print head was still moving the carriage back to the first
        position.

    - Many early video displays also required multiple character times to scroll
        the display. MS-DOS adopted CP/M's CR+LF standard to be compatible. When
        the widespread adoption of the personal computer, the tradition became
        imprinted in newer versions of DOS and Windows.

    - During the same period, Multics (1964) used only the LF character because
        CR was useful for bold and strikethrough effects. This convention was
        adopted by ISO in 1967 and later by UNIX. All UNIX based operating
        systems carried the LF flag forward, with one exception being Apple.

    - The Apple II computer, as well as Commodore, TRS-80, classic Mac OS used CR
        alone. This led to even greater confusion as files encoded on Unix, Apple,
        and Dos based operating systems were all using ASCII encoding and yet were
        frustratingly mutually incompatible.

    - Apple has since joined the Multics crowd and ends lines with LF (0A) instead
        of the old CR (0D). Microsoft has continued to use the 2 character set for
        encoding line endings, but software drivers have eliminated most of the
        prior confusion.

    </aside>
    """


for num, line in enummerate(newline_text):
    s = f"{num:>3.3}:  {line}"



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# 'future imports'
from __future__ import absolute_import

# 'Standard Library'
import sys

from collections import deque
from contextlib import closing
from dataclasses import dataclass
from logging import Logger
from os import environ as ENV, linesep as NL
from pprint import PrettyPrinter
from sys import argv, stderr, stdout

# 'package imports'
from bs4 import BeautifulSoup
# 'third party'
from requests import get, Response
from requests.exceptions import RequestException

from typing import Any, Deque, Dict, List, Tuple

""" web_basics.py - basic web scraping tools

    copyright (c) 2019 Michael Treanor
    https://www.github.com/skeptycal/autosys
    https://www.twitter.com/skeptycal
    """

try:
    import lxml as parser

    DEFAULT_PARSER = "lxml"
except:
    DEFAULT_PARSER = "html.parser"

# from autosys import *

# !---------------------------------------------- Common CONSTANTS
_debug_: bool = True
_fuzzy_: bool = True

DEFAULT_URL_HISTORY: int = 1000

log_web = Logger(__file__)
pp_web = PrettyPrinter(indent=2,
                       width=79,
                       depth=5,
                       stream=stdout,
                       compact=False)

# !---------------------------------------------- Custom Types


class WebPageError(Exception):
    """ An error occured while handling your web document request. """

    this = None


class ScrapeRulesError(Exception):
    """ An error occurred while applying rules to web scraping activities. """


class ScrapeSetError(Exception):
    """ An error occurred while managing the web scraping document set. """


@dataclass()
class ConfigDefaults:
    _debug: bool = False
    _log: bool = False and _debug
    _fuzzy: bool = True


def simple_get(url):
    """
    Attempts to get the content at `url` by making an HTTP GET request.
    If the content-type of response is some kind of HTML/XML, return the
    text content, otherwise return None.
    """
    try:
        with closing(get(url, stream=True)) as resp:
            if is_good_response(resp):
                return resp.content
            else:
                return None

    except RequestException as e:
        log_error("Error during requests to {0} : {1}".format(url, str(e)))
        return None


def is_good_response(resp):
    """
    Returns True if the response seems to be HTML, False otherwise.
    """
    content_type = resp.headers["Content-Type"].lower()
    return (resp.status_code == 200 and content_type is not None
            and content_type.find("html") > -1)


def log_error(e):
    """
    It is always a good idea to log errors.
    This function just prints them, but you can
    make it do anything.
    """
    print(e)


def get_p_tags(html):
    for p in html.select("p"):
        yield p.text


def soup(url):
    return BeautifulSoup(simple_get(url), DEFAULT_PARSER)


# !---------------------------------------------- Script Tests
def __tests__(args) -> int:
    """ Run Debug Tests for script if _debug_ = True. """

    print(f"{DEFAULT_PARSER=}")
    url = "https://realpython.com/blog/"
    raw_html = simple_get(url)
    html = soup(url)
    pprint(html)
    return 0


def __main__(args=argv[1:]) -> int:
    """ CLI script main entry point. """
    try:
        print("before")
        # raise WebPageError("test it")
        print("after")
    except:
        log_web.error()
        print("log")

    #! script testing
    if _debug_:
        __tests__(args)


if __name__ == "__main__":  # if script is loaded directly from CLI
    print("hello world")
    __main__()
""" Notes
    from tutorial ... moved out of __test__()

    raw_html = simple_get("https://realpython.com/blog/")
    print(len(raw_html))

    no_html = simple_get("https://realpython.com/blog/nope-not-gonna-find-it")

    print(no_html is None)

    # raw_html = open("contrived.html").read()
    """



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

# Reference:
# https://www.geeksforgeeks.org/python-get-google-map-image-specified-location-using-google-static-maps-api/

# Google API Key and account:
# https://developers.google.com/maps/documentation/maps-static/intro

# Python program to get a google map
# image of specified location using
# Google Static Maps API

# 'third party'
# importing required modules
import requests

# Enter your api key here
api_key = "_your_api_key_"

# url variable store url
url = "https://maps.googleapis.com/maps/api/staticmap?"

# center defines the center of the map,
# equidistant from all edges of the map.
center = "Dehradun"

# zoom defines the zoom
# level of the map
zoom = 10

# get method of requests module
# return response object
r = requests.get(url + "center =" + center + "&zoom =" + str(zoom) +
                 "&size = 400x400&key =" + api_key + "sensor = false")

# wb mode is stand for write binary mode
f = open("address of the file location ", "wb")

# r.content gives content,
# in this case gives image
f.write(r.content)

# close method of file object
# save and close the file
f.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" _version.py - version and demographic information

    Part of the `AutoSys` package - utilities for macOS apps
        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """
# 'Standard Library'
from dataclasses import dataclass
from datetime import date as _date
from pprint import pprint

from typing import Dict, List


# print(dir(_date))
def kv(k, d):
    fmt = f"{d}.{k}()"
    try:
        return eval(f"{fmt}")
    except Exception as e:
        print(f"Error: {e.args[0]:<10.15}. {fmt}")

    fmt = f"{d}.{k}"
    try:
        return eval(fmt)
    except Exception as e:
        print(f"Error: {e.args[0]:<10.15}. {fmt}")

    return "error"


class ApiSpy:
    src: Dict
    blacklist: List
    whitelist: List
    """ Get, create, use, evaluate, and document api functionality

        produce a list of outputs from various functions in a module
        this is basically an automated:
        - an api front end for python modules
        - module tester
        - documentation maker
        - use some simple datetime results to test it


        get keys from dir(module) that are not in blacklist, but are in whitelist
        - use the dunders for info
        - use doc for info
        - search online for documentation
        - try different values until the functions report something feasible

        automating this stuff will help a lot. being able to use modules more automatically would help, and autodocumentation isn't bad either ...

        check keys for values:
        - try calling f(some_default)
        - try calling f()
        - try calling f
        - take a list of inputs to use for f(x) and zip up a dict
        """
    def __init__(self, d):
        pass
        """
        - create a list of functionality
        - identify usual suspects to blacklist
        - default blacklist is starting point
        - identify or input whitelist items ... those will be thoroughly tested and never dropped

        """


_date_dict = {x: kv(x, "_date") for x in dir(_date) if not x.startswith("_")}

# print(_date_dict)

for k, v in _date_dict.items():
    print(f"{k}: {v}\n")


class Now(_date):

    date = _date.today()
    year = date.year

    def __init__(self):

        print(self.year)


# n = Now()


def what_year_is_it():
    from datetime import date

    return str(date.today().year)


if True:
    from pprint import pprint

    _intro = f"{__title__.title()} setup and version information:"
    _hr = "=" * len(_intro)
    print(_intro)
    print(_hr)
    _fd = {k: eval(k) for k in __all__}
    pprint(_fd)
    # for f in _fields:
    #     print(f"{f} - {eval(f)}")
    #     print('-' * 50)
    print()

    print(_hr)



# 'Standard Library'
import time

# 'package imports'
from selenium import webdriver

_chrome_driver: str = "/path/to/chromedriver"  # Optional argument, if not specified will search path.
_chrome_driver = ""
driver = webdriver.Chrome(_chrome_driver)
driver.get("http://www.google.com/")
time.sleep(5)  # Let the user actually see something!
search_box = driver.find_element_by_name("q")
search_box.send_keys("ChromeDriver")
search_box.submit()
time.sleep(5)  # Let the user actually see something!
while True:
    time.sleep(100)
# driver.quit()



# -*- coding: utf-8 -*-
""" Part of the `AutoSys` package
        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """
__license__ = "MIT"

# 'Standard Library'
import os
import sys

# !---------------------------------------------- String Utilities

NL: str = os.linesep


def arg_str(*args, sep=" ") -> (str):
    """ Return one string created from *args sequence.

        sep - string used in between joined elements.

        each element is converted to a string where possible, otherwise
        it is ignored and silently logged as an error
        """
    from typing import Any, List

    _: List[Any] = []
    for arg in args:
        try:  # convert args to strings
            _.append(str(arg))
        except:
            try:
                dbprint(
                    f"An argument could not be converted to a string for display - {arg}"
                )
            except:
                pass  # skip debug output if not available
            pass  # skip args that cannot be converted to strings
    return sep.join(_)


def br(n: int = 1) -> (int):
    """ Yes, this is a CLI version of a <br /> tag ...

        n : int - number of line breaks to print

        return int 0 for success else 1
        """
    try:
        print(NL * n, end="", sep="")
        return 0
    except:
        return 1


def brerr(n: int = 1) -> (int):
    """ Yes, this is a CLI version of a <br /> tag ... sent to STDERR.

        n : int - number of line breaks to print

        return int 0 for success else 1
        """
    try:
        print("\n" * n, end="", sep="", file=sys.stderr)
        return 0
    except:
        return 1


def hr(s: str = "-", n: int = 79) -> str:
    """ Print a string duplicated <n> times.

        This can act as a page break. Yes, this is a CLI version of an <hr /> tag

        s : str  - string to repeat (default '-')
        n : int  - number of times to repeat (default 79)
        """
    print(s * n)



import sys

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3
PY38 = sys.version_info[0:2] >= (3, 8)

# only Python3 is supported now ...
if PY3:
    string_types = str,
    integer_types = int,
    class_types = type,
    text_type = str
    binary_type = bytes
    from sys import maxsize as MAXSIZE
""" else:
        string_types = basestring,
        integer_types = (int, long)
        class_types = (type, types.ClassType)
        text_type = unicode
        binary_type = str

        if sys.platform.startswith("java"):
            # Jython always uses 32 bits.
            MAXSIZE = int((1 << 31) - 1)
        else:
            # It's possible to have sizeof(long) != sizeof(Py_ssize_t).
            class X(object):
                def __len__(self):
                    return 1 << 31

            try:
                len(X())
            except OverflowError:
                # 32-bit
                MAXSIZE = int((1 << 31) - 1)
            else:
                # 64-bit
                MAXSIZE = int((1 << 63) - 1)
            del X """


def force_unicode(text):
    """Takes a text (Python 2: str/unicode; Python 3: unicode) and converts to unicode

    :arg str/unicode text: the text in question

    :returns: text as unicode

    :raises UnicodeDecodeError: if the text was a Python 2 str and isn't in
        utf-8

    """
    # If it's already unicode, then return it
    if isinstance(text, six.text_type):
        return text

    # If not, convert it
    return six.text_type(text, 'utf-8', 'strict')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" Common imports, CONSTANTS, and utilities for the `AutoSys` package

    Part of the `AutoSys` package - utilities for macOS apps
        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """

# logging setup
# Copyright (C) 2012-2019 Vinay Sajip.
# Licensed to the Python Software Foundation under a contributor agreement.

# 'Standard Library'
import logging
import re
import sys

from dataclasses import Field, dataclass, field
from io import TextIOWrapper
from os import environ as ENV, linesep as NL
from sys import argv, path as PYTHON_PATH, stderr, stdout

from typing import Any, Dict, List, NamedTuple, Sequence, Tuple

# 'package imports'
# from autosys.cli.ansi_codes import AnsiCodes
# from autosys.cli.ascii_chars import *
# from autosys.cli.supports_color import SUPPORTS_COLOR
# from autosys.cli.terminal import *



class CommandLineInterfaceException(Exception):
    pass


try:
    from logging import NullHandler
except ImportError:  # pragma: no cover

    class NullHandler(logging.Handler):
        def handle(self, record):
            pass

        def emit(self, record):
            pass

        def createLock(self):
            self.lock = None


logger = logging.getLogger(__name__)
logger.addHandler(NullHandler())



""" # * From pywall - see description at the end of this file

Contains Python objects for IP and network layer datagrams/segments.

These generally take a buffer in their constructor, and parse the header fields
into class member fields so that we can access them easily.

"""

# 'future imports'
from __future__ import unicode_literals

# 'Standard Library'
import socket

from abc import ABCMeta, abstractmethod
from struct import unpack

# A list of IP Protocol numbers, taken directly from IANA.
PROTO_NUMS = {
    0: "HOPOPT",
    1: "ICMP",
    2: "IGMP",
    3: "GGP",
    4: "IPv4",
    5: "ST",
    6: "TCP",
    7: "CBT",
    8: "EGP",
    9: "IGP",
    10: "BBN-RCC-MON",
    11: "NVP-II",
    12: "PUP",
    13: "ARGUS",
    14: "EMCON",
    15: "XNET",
    16: "CHAOS",
    17: "UDP",
    18: "MUX",
    19: "DCN-MEAS",
    20: "HMP",
    21: "PRM",
    22: "XNS-IDP",
    23: "TRUNK-1",
    24: "TRUNK-2",
    25: "LEAF-1",
    26: "LEAF-2",
    27: "RDP",
    28: "IRTP",
    29: "ISO-TP4",
    30: "NETBLT",
    31: "MFE-NSP",
    32: "MERIT-INP",
    33: "DCCP",
    34: "3PC",
    35: "IDPR",
    36: "XTP",
    37: "DDP",
    38: "IDPR-CMTP",
    39: "TP++",
    40: "IL",
    41: "IPv6",
    42: "SDRP",
    43: "IPv6-Route",
    44: "IPv6-Frag",
    45: "IDRP",
    46: "RSVP",
    47: "GRE",
    48: "DSR",
    49: "BNA",
    50: "ESP",
    51: "AH",
    52: "I-NLSP",
    53: "SWIPE (deprecated)",
    54: "NARP",
    55: "MOBILE",
    56: "TLSP",
    57: "SKIP",
    58: "IPv6-ICMP",
    59: "IPv6-NoNxt",
    60: "IPv6-Opts",
    62: "CFTP",
    64: "SAT-EXPAK",
    65: "KRYPTOLAN",
    66: "RVD",
    67: "IPPC",
    69: "SAT-MON",
    70: "VISA",
    71: "IPCV",
    72: "CPNX",
    73: "CPHB",
    74: "WSN",
    75: "PVP",
    76: "BR-SAT-MON",
    77: "SUN-ND",
    78: "WB-MON",
    79: "WB-EXPAK",
    80: "ISO-IP",
    81: "VMTP",
    82: "SECURE-VMTP",
    83: "VINES",
    84: "TTP",
    84: "IPTM",
    85: "NSFNET-IGP",
    86: "DGP",
    87: "TCF",
    88: "EIGRP",
    89: "OSPFIGP",
    90: "Sprite-RPC",
    91: "LARP",
    92: "MTP",
    93: "AX.25",
    94: "IPIP",
    95: "MICP (deprecated)",
    96: "SCC-SP",
    97: "ETHERIP",
    98: "ENCAP",
    100: "GMTP",
    101: "IFMP",
    102: "PNNI",
    103: "PIM",
    104: "ARIS",
    105: "SCPS",
    106: "QNX",
    107: "A/N",
    108: "IPComp",
    109: "SNP",
    110: "Compaq-Peer",
    111: "IPX-in-IP",
    112: "VRRP",
    113: "PGM",
    115: "L2TP",
    116: "DDX",
    117: "IATP",
    118: "STP",
    119: "SRP",
    120: "UTI",
    121: "SMP",
    122: "SM",
    123: "PTP",
    124: "ISIS over IPv4",
    125: "FIRE",
    126: "CRTP",
    127: "CRUDP",
    128: "SSCOPMCE",
    129: "IPLT",
    130: "SPS",
    131: "PIPE",
    132: "SCTP",
    133: "FC",
    134: "RSVP-E2E-IGNORE",
    135: "Mobility Header",
    136: "UDPLite",
    137: "MPLS-in-IP",
    138: "manet",
    139: "HIP",
    140: "Shim6",
    141: "WESP",
    142: "ROHC",
}


def payload_builder(payload_buff, protocol):
    """If `protocol` is supported, builds packet object from buff."""
    if protocol == socket.IPPROTO_TCP:
        return TCPPacket(payload_buff)
    elif protocol == socket.IPPROTO_UDP:
        return UDPPacket(payload_buff)
    else:
        return None


def to_tuple(ippacket, flip=False):
    """Create a tuple from a TCP packet.

    The flip argument flips the source and destination port, so that they will
    be consistent between ingress and egress.

    """
    payload = ippacket.get_payload()
    if type(payload) is TCPPacket and not flip:
        tup = (
            ippacket.get_src_ip(),
            payload.get_src_port(),  # remote
            ippacket.get_dst_ip(),
            payload.get_dst_port(),
        )  # local
        return tup
    elif type(payload) is TCPPacket and flip:
        tup = (
            ippacket.get_dst_ip(),
            payload.get_dst_port(),  # remote
            ippacket.get_src_ip(),
            payload.get_src_port(),
        )  # local
    else:
        tup = None
    return tup


def proto_to_string(proto):
    """Convert protocol number to a string."""
    return PROTO_NUMS.get(proto, "unknown")


class Packet(object):
    """Base class for all packets"""

    __metaclass__ = ABCMeta

    @abstractmethod
    def get_header_len(self):
        pass

    @abstractmethod
    def get_data_len(self):
        pass


class TransportLayerPacket(Packet):
    """Base class packets at the transport layer """

    __metaclass__ = ABCMeta

    @abstractmethod
    def get_body(self):
        pass


class IPPacket(Packet):
    """Base class for all packets"""

    def __init__(self, buf):
        """Create packet from raw data."""
        self.buf = buf
        self._src_ip = socket.inet_ntoa(buf[12:16])
        self._dst_ip = socket.inet_ntoa(buf[16:20])
        # Internal Header Length, in bytes
        self._ihl = (unpack("!B", buf[0])[0] & 0xF) * 4
        self._proto = unpack("!B", buf[9])[0]
        self._payload = payload_builder(buf[self._ihl :], self._proto)

    def get_src_ip(self):
        return self._src_ip

    def get_dst_ip(self):
        return self._dst_ip

    def get_protocol(self):
        return self._proto

    def get_payload(self):
        return self._payload

    def get_header_len(self):
        return self._ihl

    def get_data_len(self):
        return len(self.buf) - self._ihl

    def __unicode__(self):
        return "IP Packet %s => %s, proto=%s" % (
            self._src_ip,
            self._dst_ip,
            proto_to_string(self._proto),
        )


class TCPPacket(TransportLayerPacket):
    """TCP Packet object."""

    def __init__(self, buff):
        self._parse_header(buff)

    def _parse_header(self, buff):
        self._src_port, self._dst_port = unpack("!HH", buff[0:4])
        self._seq_num, self._ack_num = unpack("!II", buff[4:12])
        flags, self._win_size = unpack("!HH", buff[12:16])
        self._data_offset = flags & 0xF000
        self.flag_ns = flags & 0x0100
        self.flag_cwr = flags & 0x0080
        self.flag_ece = flags & 0x0040
        self.flag_urg = flags & 0x0020
        self.flag_ack = flags & 0x0010
        self.flag_psh = flags & 0x0008
        self.flag_rst = flags & 0x0004
        self.flag_syn = flags & 0x0002
        self.flag_fin = flags & 0x0001
        self._checksum, self._urg_ptr = unpack("!HH", buff[16:20])
        # can be parsed later if we care:
        self._options = buff[20 : (self._data_offset * 4)]
        self._total_length = len(buff)
        self._body = buff[self.get_header_len() :]

    def get_header_len(self):
        return self._data_offset * 4

    def get_data_len(self):
        return self._total_length - self.get_header_len()

    def get_src_port(self):
        return self._src_port

    def get_dst_port(self):
        return self._dst_port

    def get_body(self):
        return str(self._body)

    def __unicode__(self):
        """Returns a printable version of the TCP header"""
        return "TCP from %d to %d" % (self._src_port, self._dst_port)


class UDPPacket(TransportLayerPacket):
    """UDP Packet object."""

    def __init__(self, buff):
        self._parse_header(buff)

    def _parse_header(self, buff):
        self._src_port, self._dst_port = unpack("!HH", buff[0:4])
        self._length, self._checksum = unpack("!HH", buff[4:8])
        self._total_length = len(buff)
        self._body = buff[self.get_header_len() :]

    def get_header_len(self):
        return 8

    def get_data_len(self):
        return self._total_length - self.get_header_len()

    def get_src_port(self):
        return self._src_port

    def get_dst_port(self):
        return self._dst_port

    def get_body(self):
        return str(self._body)

    def __unicode__(self):
        """Returns a printable version of the UDP header"""
        return "UDP from %d to %d" % (self._src_port, self._dst_port)


""" #*#####################################################################

PyWall
======

A Python firewall: Because slow networks are secure networks.


Installation
------------

This section assumes that you are installing this program on Ubuntu 14.04 LTS.
This firewall should work on other Linux systems, but safety not guaranteed.

First, install the required packages. On Ubuntu, these are `iptables`, `python`,
`python-pip`, `build-essential`, `python-dev`, and
`libnetfilter-queue-dev`. Next, use `pip2` to install the project dependencies,
which can be found in `requirements.txt`.

The commands for both these operations are:

    sudo apt-get install python python-pip iptables build-essential python-dev libnetfilter-queue-dev
    pip install --user -r requirements.txt


Running
-------

The main file is `main.py`, which needs to be run as root to modify IPTables.
Additionally, main needs to receive a JSON configuration file as its first
argument. If running with the example configuration, the command is:

`sudo python2 main.py examples/example.json`

To stop PyWall, press Control-C.


Troubleshooting
---------------

PyWall should undo its changes to IPTables after exiting. However, if you are
unable to access the internet after exiting PyWall, view existing
IPTables rules with `sudo iptables -nL`. If a rule with the target chain
`NFQueue` lingers, delete it with
`sudo iptables -D INPUT -j NFQUEUE --queue-num [undesired-queue-number]`.

For INPUT rules, the command is `sudo iptables -D INPUT -j NFQUEUE --queue-num 1`.
For OUTPUT rules, the command is `sudo iptables -D OUTPUT -j NFQUEUE --queue-num 2`.

In case PyWall gives a message that another application has the xtables lock,
Control-C the server, ensure that all the IPTables rules are cleared, and
restart PyWall.

"""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" Part of the `AutoSys` package - utilities for macOS apps
        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """

# 'Standard Library'
from collections import OrderedDict

# These colors are from Tableau
_TABLEAU_COLORS = (
    ("blue", "#1f77b4"),
    ("orange", "#ff7f0e"),
    ("green", "#2ca02c"),
    ("red", "#d62728"),
    ("purple", "#9467bd"),
    ("brown", "#8c564b"),
    ("pink", "#e377c2"),
    ("gray", "#7f7f7f"),
    ("olive", "#bcbd22"),
    ("cyan", "#17becf"),
)

# Normalize name to "tab:<name>" to avoid name collisions.
TABLEAU_COLORS = OrderedDict(
    ("tab:" + name, value) for name, value in _TABLEAU_COLORS
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" AutoSys loguru log wrapper used to set default log level
    of default log.

    Use an environment variable set in .bashprofile (or .zshrc, etc):

    ```sh
    LOGURU_LEVEL=TRACE
    ```

    or place a constant below in place of "TRACE":
    ```
    LOGURU_LEVEL: str = environ.get("LOGURU_LEVEL", "TRACE")
    ```

    then run this from the main script
    ```
    from log import logger
    ```
    ---
    System utilities for Python.

    Part of the [AutoSys][1] package

    Copyright (c) 2018 [Michael Treanor][2]

    AutoSys is licensed under the [MIT License][3]

    [1]: https://www.github.com/skeptycal/autosys
    [2]: https://www.twitter.com/skeptycal
    [3]: https://opensource.org/licenses/MIT
    """

__version__ = '0.1.0'

import atexit as _atexit
import sys as _sys
from loguru import _Logger, _Core

try:
    _debug_
except NameError:
    _debug_: bool = True

# original class init method:
# _Logger.__init__(self, core, exception, depth, record, lazy, colors, raw, capture, patcher, extra)

# TODO - pass in list of handlers instead of just one flag
class AutoSysLogger(_Logger):
    """ Extra Features ...

        If level is set to "NONE", the logger will still load but no handlers
        will be added. """

    LOGURU_LEVEL: str = ""
    DEFAULT_PROD_VALUE: str = "SUCCESS"  # for no logging, use NONE
    DEFAULT_DEV_VALUE: str = "TRACE"
    LOGGING: bool = False

    def __init__(
        self,
        core=_Core(),
        exception=None,
        depth=0,
        record=False,
        lazy=False,
        colors=True,
        raw=False,
        capture=True,
        patcher=None,
        extra={},
        debug=False,
        # handlers = {}
    ):
        super().__init__(_Core(), None, 0, False, False, False, False, True, None, {})
        self._autosys_defaults(debug=debug)

    def _autosys_defaults(self, debug: bool = False):
        """ Run once from self.__init__. """

        # set default severity
        try:
            if debug:
                self.LOGURU_LEVEL = self.DEFAULT_DEV_VALUE
            else:
                self.LOGURU_LEVEL = self.DEFAULT_PROD_VALUE
        except:
            self.LOGURU_LEVEL = self.DEFAULT_PROD_VALUE

        if self.LOGURU_LEVEL != "NONE":
            # set LOGGING flag if self.LOGURU_LEVEL is specified
            self.LOGGING = True

            from os import environ as _env

            # check for environment variable that overrides default value
            self.LOGURU_LEVEL = _env.get("LOGURU_LEVEL", self.LOGURU_LEVEL)

            # remove default handler
            try:
                self.remove(0)
            except ValueError:
                pass

            # setup new handler with correct severity
            self.add(_sys.stderr, level=self.LOGURU_LEVEL)
            # self.info(f"Logging is on. Severity level set to {self.LOGURU_LEVEL}")
            del _env


__all__ = ["logger"]


logger = AutoSysLogger(
    core=_Core(),
    exception=None,
    depth=0,
    record=False,
    lazy=False,
    colors=True,
    raw=False,
    capture=True,
    patcher=None,
    extra={},
    debug=_debug_,
)


logger.info(f"Logging is on. Severity level set to '{logger.LOGURU_LEVEL}'")

# this was the original default handler
# if _defaults.LOGURU_AUTOINIT and _sys.stderr:
#     logger.add(_sys.stderr)

_atexit.register(logger.remove)



# 'Standard Library'
import threading
import time

# 'package imports'
import pyperclip


def is_url_but_not_bitly(url):
    if url.startswith("http://") and not "bit.ly" in url:
        return True
    return False


def print_to_stdout(clipboard_content):
    print("Found url: %s" % str(clipboard_content))


class ClipboardWatcher(threading.Thread):
    def __init__(self, predicate, callback, pause=5.0):
        super(ClipboardWatcher, self).__init__()
        self._predicate = predicate
        self._callback = callback
        self._pause = pause
        self._stopping = False

    def run(self):
        recent_value = ""
        while not self._stopping:
            tmp_value = pyperclip.paste()
            if tmp_value != recent_value:
                recent_value = tmp_value
                if self._predicate(recent_value):
                    self._callback(recent_value)
            time.sleep(self._pause)

    def stop(self):
        self._stopping = True


def main():
    watcher = ClipboardWatcher(is_url_but_not_bitly, print_to_stdout, 5.0)
    watcher.start()
    while True:
        try:
            print("Waiting for changed clipboard...")
            time.sleep(10)
        except KeyboardInterrupt:
            watcher.stop()
            break


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Python program to print
# colored text and background
# https://www.geeksforgeeks.org/print-colors-python-terminal/

from typing import Dict, List

MAIN = "\u001b[38; 5; 229m"
RESET = "\u001b[0m"

ATTR: Dict[str, str] = {
    "0": "\u001b[0m",
    "Reset": "\u001b[0m",
    "1": "\u001b[1m",
    "Bold": "\u001b[1m",
    "2": "\u001b[2m",
    "Dim": "\u001b[2m",
    "3": "\u001b[3m",
    "Italic": "\u001b[3m",
    "4": "\u001b[4m",
    "Underline": "\u001b[4m",
    "7": "\u001b[7m",
    "Reverse": "\u001b[7m",
    "8": "\u001b[8m",
    "Conceal": "\u001b[8m",
}
# Generator resources:
# ATTRIBUTE_CODES: List[int] = [0, 1, 2, 3, 4, 7, 8]
# ATTR: Dict[int, str] = {k: '\u001b['+str(k)+'m' for k in [0, 1, 2, 3, 4, 7, 8]}


def print_format_table():
    """
    prints table of formatted text format options
    """
    for style in range(8):
        for fg in range(30, 37):
            s1 = ""
            for bg in range(40, 47):
                format = ";".join([str(style), str(fg), str(bg)])
                s1 += "\x1b[%sm %s \x1b[0m" % (format, format)
            print(s1)
        print("\n")


def print_attributes_table():
    """
    prints table of formatted text attribute options
    """
    for style in [0, 1, 2, 3, 4, 7, 8]:
        print(MAIN, style, " : ", RESET, end="")
        for fg in range(30, 37):
            s1 = ""
            # for bg in range(40, 47):
            format = ";".join([str(style), str(fg), str(40)])
            s1 += "\x1b[%sm %s \x1b[\t" % (format, format)
            print(s1, end="")
        print("\n")


print_format_table()
# print_attributes_table()
print(RESET, MAIN, ATTR)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" table_dict

    print a pretty table from a dictionary

    ---
    Part of the [AutoSys][1] package

    Copyright (c) 2018 [Michael Treanor][2]

    AutoSys is licensed under the [MIT License][3]

    [1]: https://www.github.com/skeptycal/autosys
    [2]: https://www.twitter.com/skeptycal
    [3]: https://opensource.org/licenses/MIT
    """

from typing import Dict


def table_dict(_vars: Dict, width=60, divider=": ", indent=2, key_size=15):
    min_width: int = key_size
    print("-" * width)
    key_size = ((key_size := width // 4) < min_width) or min_width
    print(key_size)
    value_size = width - key_size - indent - len(divider) - 1
    for k, v in _vars.items():
        val = str(v)[:value_size]
        print(f"{indent*' '}{k:<{key_size}.{key_size}}{divider}{val}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" as_json.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal

# 'Standard Library'
# * ##########################################################################
# features in this module:
#   _json_translation_table - print python to json translation tables
#   json_read               - open and read data into python object
#   json_write              - write json object to file and sort if needed
#   json_sort               - sort list of python files as json objects
#   json_minify             - compact json files and remove comments
#   json_pretty_print       - format 'pretty print' style
# * ##########################################################################
import fileinput
import os
import pathlib
import re
import sys

from io import StringIO

from flask import current_app, request

from typing import Any, Dict, List, Tuple, Union

try:
    import ujson as json  # Speedup if present; no big deal if not

    JSON_PARSER = "ujson"
except ImportError:
    import json

    JSON_PARSER = "json"
# Reference: https://pypi.org/project/ujson/
# Use speedup if available
# scanstring = c_scanstring or py_scanstring

# fix import path for non standard libraries
sys.path.append(os.path.dirname(os.path.realpath(__file__)))

# * ##########################################################################
# constants
DEBUG_FLAG: bool = True
ENCODING: str = "utf8"

# * ##########################################################################
# general functions


def db_print(*args, **kwargs):
    """ db_print - print to console only if debug flag is set """
    if DEBUG_FLAG:
        print(*args, **kwargs)


def is_file(file_name: str) -> bool:
    """ Return true if file_name exists and is a file. """
    if pathlib.Path(file_name).is_file():
        return True
    else:
        return False


# * ##########################################################################
# json functions
class JSON_file(object):
    io = StringIO()
    file_name: str = ""
    json_data: Any = None
    path_name = ""
    pattern_name = ""
    encoding: str = ENCODING

    def __init__(self, *args, **kwargs):
        self.encoding = ENCODING
        super().__init__(*args, **kwargs)

    def set_file(self, json_file: str) -> bool:
        if is_file(json_file):
            self.file_name = json_file
            return True
        else:
            return False

    def error_handler(self, e: Exception):
        # TODO add error handling ...
        return False

    def _json_translation_table(self) -> (str):
        # Reference: https://docs.python.org/3/library/json.html#json-to-py-table
        """
            _json_translation_table : return json -> python translation table
                result (str)        : string containing formatted table
            """
        result = """     json translations for python

            Reference:
                https://docs.python.org/3/library/json.html

                ╔═══════════════════════════════╗
                ║      class json.JSONDecoder     ║
                ║      (Simple JSON decoder)      ║
                ╠═════════════════╦═════════════╣
                ║  JSON            ║  Python      ║
                ╠═════════════════╬═════════════╣
                ║  object          ║  dict        ║
                ║  array           ║  list        ║
                ║  string          ║  str         ║
                ║  number (int)    ║  int         ║
                ║  number (real)   ║  float       ║
                ║  true            ║  True        ║
                ║  false           ║  False       ║
                ║  null            ║  None        ║
                ╚═════════════════╩═════════════╝

                ╔═══════════════════════════════╗
                ║      class json.JSONEncoder     ║
                ║      (Simple JSON encoder)      ║
                ╠═════════════════╦═════════════╣
                ║  Python          ║  JSON        ║
                ╠═════════════════╬═════════════╣
                ║  dict            ║  object      ║
                ║  list, tuple     ║  array       ║
                ║  str             ║  string      ║
                ║  int,float,etc   ║  number      ║
                ║  True            ║  true        ║
                ║  False           ║  false       ║
                ║  None            ║  null        ║
                ╚═════════════════╩═════════════╝
        """
        return result

    def json_comments(self, comments):
        if not comments:
            self.json_data = json.loads("".join(line for line in data_file
                                                if not line.startswith("//")))

    def json_read(self) -> (bool):
        """
            json_read: load a json file and remove comments if needed
                (comments are not officially allowed in json format)

            parameter:
                json_file (str)     : file to load
                comments (bool)     : comments are allowed? (default False)
            return:
                result (object)     : on success - python json object from json_file
                                    : error code if failure
            """
        try:
            with open(self.file_name, "r") as data_file:
                self.json_data = json.load(self.file_name)
                return True
        except (OSError, IOError, ValueError) as e:
            return False

    def json_write(self) -> (bool):
        """
            json_write: write json object to file and sort if needed

            parameter:
                json_file (str)     : file name used to write json_data
                json_data (object)  : json data as python object
                is_sorted (bool)    : sort keys alphabetically (default True)
            return:
                result (int)     : 0 for success else error code
            """
        try:
            with open(self.file_name, "w",
                      encoding=self.encoding) as data_file:
                data_file.write(self.json_data)
            return True
        except (OSError, IOError, ValueError) as e:
            self.error_handler(e)
            return False

    def json_format(
        self,
        comments: bool = False,
        sort_keys: bool = True,
        separators: Tuple[str, str] = (", ", ": "),
        indent: Union[int, str] = 4,
        ensure_ascii: bool = False,
        skipkeys: bool = False,
        check_circular: bool = True,
    ) -> (bool):
        try:
            self.json_data = json.dumps(
                self.file_name,
                sort_keys=sort_keys,
                separators=separators,
                indent=indent,
                ensure_ascii=ensure_ascii,
                skipkeys=skipkeys,
                check_circular=check_circular,
            )
            return True
        except (OSError, IOError, ValueError) as e:
            self.error_handler(e)
            return False

    def json_sort(self, comments: bool = False) -> (bool):
        """
            json_sort: alphabetize json files and remove comments if needed

            parameter:
                comments (bool)     : comments are allowed? (default False)
            return:
                result (bool)       : True if successful else False
            """
        return self.json_format(self, comments=True)

    def json_minify(self, comments: bool = False) -> (bool):
        """
            json_minify: compact json data and remove comments

            parameter:
                comments (bool)     : comments are allowed? (default False)
            return:
                result (bool)       : True if successful else False
            """
        return self.json_format(
            self,
            comments=False,
            sort_keys=False,
            separators=(",", ":"),
            indent=None,
        )

    def json_pretty_print(self, comments: bool = False) -> (str):
        """
            json_pretty_print: format json for pretty printing

            parameter:
                comments (bool)     : comments are allowed? (default False)
            return:
                result (str)        : json file as formatted string
            """
        return self.json_format(self, comments=True, sort_keys=False)


# * ##########################################################################

# Reference: https://gist.github.com/liftoff/ee7b81659673eca23cd9fc0d8b8e68b7


def ultrajsonify(*args, **kwargs):
    """This function reimplements ``flask.json.jsonify``
    using ``ujson.dumps`` instead of ``json.dumps``.
    """
    indent = 0
    ensure_ascii = current_app.config.get("JSON_AS_ASCII", True)
    mimetype = current_app.config.get("JSONIFY_MIMETYPE", "application/json")

    if (current_app.config["JSONIFY_PRETTYPRINT_REGULAR"]
            and not request.is_xhr):
        indent = 2

    if args and kwargs:
        raise ValueError(
            "ultrajsonify behavior undefined when passed both args and kwargs")
    elif len(args) == 1:
        data = args[0]
    else:
        data = args or kwargs

    return current_app.response_class(
        dumps(data, indent=indent, ensure_ascii=ensure_ascii),
        mimetype=mimetype,
    )


def remove_comments(json_like):
    """
    Removes C-style comments from *json_like* and returns the result.  Example::

        >>> test_json = '''\
        {
            "foo": "bar", // This is a single-line comment
            "baz": "blah" /* Multi-line
            Comment */
        }'''
        >>> remove_comments('{"foo":"bar","baz":"blah",}')
        '{\n    "foo":"bar",\n    "baz":"blah"\n}'
    """
    comments_re = re.compile(
        r'//.*?$|/\*.*?\*/|\'(?:\\.|[^\\\'])*\'|"(?:\\.|[^\\"])*"',
        re.DOTALL | re.MULTILINE,
    )

    def replacer(match):
        s = match.group(0)
        if s[0] == "/":
            return ""
        return s

    return comments_re.sub(replacer, json_like)


def remove_trailing_commas(json_like):
    """
    Removes trailing commas from *json_like* and returns the result.  Example::

        >>> remove_trailing_commas('{"foo":"bar","baz":["blah",],}')
        '{"foo":"bar","baz":["blah"]}'
    """
    trailing_object_commas_re = re.compile(
        r'(,)\s*}(?=([^"\\]*(\\.|"([^"\\]*\\.)*[^"\\]*"))*[^"]*$)')
    trailing_array_commas_re = re.compile(
        r'(,)\s*\](?=([^"\\]*(\\.|"([^"\\]*\\.)*[^"\\]*"))*[^"]*$)')
    # Fix objects {} first
    objects_fixed = trailing_object_commas_re.sub("}", json_like)
    # Now fix arrays/lists [] and return the result
    return trailing_array_commas_re.sub("]", objects_fixed)


# * ############################################################################


def to_json(text: str) -> (Dict):
    """ Convert text string to json """
    text.rstrip(",")
    rawfile = (line for line in text if line[0] not in ["/", "#"])
    raise NotImplementedError


def sort_json(json_files: List[str],
              comments: bool = False,
              delim: str = ",") -> (List[str]):
    """
        sort_json: alphabetize json files and remove comments if needed

        parameter:
            json_files (list)   : 1 or more files to sort
            comments (bool)     : comments are allowed? (default False)
            delim (str)         : delimiter used for input files (default ",")
        return:
            result (list)       : list of failures (empty = success)
        """
    from pathlib import Path

    result = []
    for file in json_files:
        p = Path(file)
        try:
            with p.open(mode="r") as file:
                # read file contents into buffer 'raw_file'
                raw_file = "\n".join([
                    line.strip() for line in file
                    if not line.startswith("#") and not line.startswith("//")
                ])
                alphabetized = json.loads(raw_file)
                with open(file, "w") as w:
                    w.write(
                        json.dumps(
                            alphabetized,
                            separators=(",", ": "),
                            indent=4,
                            sort_keys=True,
                            skipkeys=True,
                        ))
        except OSError:
            # if unable to open file, add current file to error list result
            result.append(file)
    return result


if __name__ == "__main__":
    # * Used if run as CLI utility
    arg = ""
    args = sys.argv[1:]
    # * TEST DATA
    # args = ["/Volumes/Data/skeptycal/bin/utilities/py_imports/test.json"]
    # args = ["/Volumes/Data/skeptycal/Library/'Application Support'/Code/User/settings.json"]
    print(args)
    print(PURPLE)
    print("Output for Json_Sort module:")
    print("MIT license  |  copyright (c) 2018 Michael Treanor")
    print("<https://www.github.com/skeptycal")
    print(BLUE)
    print("List of json files: ")
    print(HEADER, args)
    print()
    result = sort_json(args)
    if result:
        for s in result:
            print(BLUE, "The file ", HEADER, s, sep="")
            print(BLUE, " could not be processed.", sep="")
    else:
        print(BLUE, "Command was successful.", sep="")
    print()

if False and __name__ == "__main__":
    json_out = ""
    for line in fileinput.input():  # Read it all in
        json_out += line
    almost_json = remove_comments(json_out)  # Remove comments
    proper_json = remove_trailing_commas(almost_json)  # Remove trailing commas
    validated = json.loads(proper_json)  # We now have parseable JSON!
    print(json.dumps(validated, indent=4))

# CLI testing
if False and __name__ == "__main__":  # If run as CLI utility
    args: List[str] = []
    TEST_FILE: str = "/Volumes/Data/skeptycal/bin/utilities/python/test.json"

    j: JSON_file = JSON_file()
    j.file_name = TEST_FILE
    print(j.file_name)

    # if j.json_read(TEST_FILE):
    #     j.json_pretty_print()
    #     print(j.json_data)

    class arg_choice(object):
        name: str = ""
        args: List[str] = []

        def __init__(self, *args, **kwargs):
            self.name = sys.argv[0]
            self.args = sys.argv[1:]
            super().__init__(*args, **kwargs)

        def parse(self):
            for arg in self.args:
                self.indirect(arg)

        def indirect(self, i):
            method_name = "number_" + str(i)
            method = getattr(self, method_name, lambda: "Invalid")
            db_print("indirect method_name: ", method_name)
            db_print("indirect method: ", method)
            return method()

        def number_0(self):
            return "zero"

        def number_1(self):
            return "one"

        def number_2(self):
            return "two"

    if len(sys.argv) > 1:
        args = sys.argv[1:]
    else:
        args = [TEST_FILE.__str__()]

    db_print()
    db_print("args: ", args[:])
    db_print("is_file({}): {}".format(args[0], is_file(args[0])))

    a: object = arg_choice()
    a.parse()

    print("a.indirect: ", a.indirect(1))

    db_print()
    # db_print (json_pretty_print(args[0]))
    db_print()

    # result = json_sort(args)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""PageRank algorithm with explicit number of iterations.

Returns
-------
ranking of nodes (pages) in the adjacency matrix

(Reference: https://en.wikipedia.org/wiki/PageRank)
"""
""" Part of the `AutoSys` package
        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """
__license__ = "MIT"


# 'third party'
import numpy as np


def pagerank(M, num_iterations: int = 100, d: float = 0.85):
    """PageRank: The trillion dollar algorithm.

    Parameters
    ----------
    M : numpy array
        adjacency matrix where M_i,j represents the link from 'j' to 'i', such that for all 'j'
        sum(i, M_i,j) = 1
    num_iterations : int, optional
        number of iterations, by default 100
    d : float, optional
        damping factor, by default 0.85

    Returns
    -------
    numpy array
        a vector of ranks such that v_i is the i-th rank from [0, 1],
        v sums to 1

    """
    N = M.shape[1]
    v = np.random.rand(N, 1)
    v = v / np.linalg.norm(v, 1)
    M_hat = d * M + (1 - d) / N
    for i in range(num_iterations):
        v = M_hat @ v
    return v


M = np.array(
    [
        [0, 0, 0, 0, 1],
        [0.5, 0, 0, 0, 0],
        [0.5, 0, 0, 0, 0],
        [0, 1, 0.5, 0, 0],
        [0, 0, 0.5, 1, 0],
    ]
)
v = pagerank(M, 100, 0.85)



#!/usr/bin/env python
# 'Standard Library'
import errno
import os
import tempfile

from hashlib import md5


class _FileCacheError(Exception):
    """Base exception class for FileCache related errors"""


class _FileCache(object):
    DEPTH = 3

    def __init__(self, root_directory=None):
        self._InitializeRootDirectory(root_directory)

    def Get(self, key):
        path = self._GetPath(key)
        if os.path.exists(path):
            with open(path) as f:
                return f.read()
        else:
            return None

    def Set(self, key, data):
        path = self._GetPath(key)
        directory = os.path.dirname(path)
        if not os.path.exists(directory):
            os.makedirs(directory)
        if not os.path.isdir(directory):
            raise _FileCacheError(
                "%s exists but is not a directory" % directory
            )
        temp_fd, temp_path = tempfile.mkstemp()
        temp_fp = os.fdopen(temp_fd, "w")
        temp_fp.write(data)
        temp_fp.close()
        if not path.startswith(self._root_directory):
            raise _FileCacheError(
                "%s does not appear to live under %s"
                % (path, self._root_directory)
            )
        if os.path.exists(path):
            os.remove(path)
        os.rename(temp_path, path)

    def Remove(self, key):
        path = self._GetPath(key)
        if not path.startswith(self._root_directory):
            raise _FileCacheError(
                "%s does not appear to live under %s"
                % (path, self._root_directory)
            )
        if os.path.exists(path):
            os.remove(path)

    def GetCachedTime(self, key):
        path = self._GetPath(key)
        if os.path.exists(path):
            return os.path.getmtime(path)
        else:
            return None

    def _GetUsername(self):
        """Attempt to find the username in a cross-platform fashion."""
        try:
            return (
                os.getenv("USER")
                or os.getenv("LOGNAME")
                or os.getenv("USERNAME")
                or os.getlogin()
                or "nobody"
            )
        except (AttributeError, IOError, OSError):
            return "nobody"

    def _GetTmpCachePath(self):
        username = self._GetUsername()
        cache_directory = "python.cache_" + username
        return os.path.join(tempfile.gettempdir(), cache_directory)

    def _InitializeRootDirectory(self, root_directory):
        if not root_directory:
            root_directory = self._GetTmpCachePath()
        root_directory = os.path.abspath(root_directory)
        try:
            os.mkdir(root_directory)
        except OSError as e:
            if e.errno == errno.EEXIST and os.path.isdir(root_directory):
                # directory already exists
                pass
            else:
                # exists but is a file, or no permissions, or...
                raise
        self._root_directory = root_directory

    def _GetPath(self, key):
        try:
            hashed_key = md5(key.encode("utf-8")).hexdigest()
        except TypeError:
            hashed_key = md5.new(key).hexdigest()

        return os.path.join(
            self._root_directory, self._GetPrefix(hashed_key), hashed_key
        )

    def _GetPrefix(self, hashed_key):
        return os.path.sep.join(hashed_key[0 : _FileCache.DEPTH])



from os import linesep as NL

from typing import Dict, Sequence


def table_print(data: (Dict, Sequence), **kwargs):
    tmp: List = []
    if isinstance(data, dict):
        tmp.extend(
            [f"{str(k):<15.15} :  {repr(v):<45.45}" for k, v in data.items()])
    elif isinstance(data, (list, tuple, set)):
        for x in data:
            try:
                tmp.append(f"{str(x):<15.15} :  {repr(f'{x}'):<45.45}")
            except:
                tmp.append(f"{str(x)}")
    else:
        raise TypeError('Parameter must be an iterable Mapping or Sequence.')
    print(NL.join(tmp), **kwargs)



# 'Standard Library'
from pathlib import Path
from traceback import format_tb

# 'package imports'
from dev.debug import dbprint

from typing import List

_debug_: bool = True  # dev mode


class FrozenError(ValueError):
    pass


class Leaf:
    """ #### qData container for Tree Structures.

        Data can be any valid object. Name and filename are optional strings.

        If <frozen> = True, no changes can be made to the data after
        instantiation. If <quiet> = True, FrozenErrors are ignored
        and the program continues.
        """

    def __init__(
        self,
        data: object,
        name: str = "",
        filename: str = "",
        quiet: bool = True,
        frozen: bool = False,
    ):
        self._data = data
        self._filename: str = filename
        self._name: str = name

        # freeze private values if frozen = True
        self._quiet: bool = quiet
        self._frozen: bool = frozen

        # super().__init__()

    def __str__(self):
        return str(self._data)

    def summary(self):
        print(
            f"data: {self}\n  (name:{self.name:<1.10}, file: {bool(self.filename)}, quiet: {self.quiet}, frozen: {self.frozen})"
        )

    def _frozen_error(self):
        raise FrozenError(
            "Changes are not allowed. The data for this leaf is frozen!"
        )

    # !----------------------------- Leaf properties

    @property
    def name(self) -> str:
        return self._name

    @property
    def quiet(self) -> bool:
        return self._quiet

    @property
    def filename(self) -> str:
        return self._filename

    @property
    def data(self):
        return self._data

    @property
    def frozen(self) -> bool:
        return self._frozen

    @name.setter
    def name(self, name: str) -> (None, Exception):
        if self.frozen:
            if not self.quiet:
                raise FrozenError
        else:
            if isinstance(name, str):
                self._name = name
            else:
                return TypeError("The type for <name> must be <str>.")

    @filename.setter
    def filename(self, filename: str) -> (None, Exception):
        if self.frozen:
            if not self.quiet:
                raise FrozenError
        else:
            if Path(filename).exists():
                dbprint(f"filename ({filename}) on leaf {self} exists.")
            else:
                try:
                    Path(filename).touch()
                except IOError as e:
                    dbprint(e)
                    return e
            self._filename = filename
            dbprint(f"filename on leaf {self} set to {self.get_name()}")
            return None

    @data.setter
    def data(self, data):
        # TODO --> perhaps add some data validation ...
        if self.frozen:
            if not self.quiet:
                raise FrozenError
        else:
            self._data = data

    @frozen.setter
    def frozen(self, frozen: bool = False):
        if self.frozen:
            if not self.quiet:
                raise FrozenError
        else:
            self._frozen = frozen

    @quiet.setter
    def quiet(self, quiet: bool = True):
        if self.frozen:
            if not quiet:
                raise FrozenError
        else:
            self._quiet = quiet


l = Leaf("test", name="testname", filename="eof", quiet=True, frozen=False)
if _debug_:
    l.summary()
    l.data = "stuff"
    print("-- changed 'data' to 'stuff'")
    l.summary()
    l.frozen = True
    print("-- changed 'frozen' to 'True'")
    l.summary()
    l.data = "this"
    print("-- changed 'data' to 'this' (no effect?)")
    l.summary()

"""
    class Tree:
        def __init__(self, data=None, left=None, right=None):
            self.top = Leaf(data=data, name=name, file=file)
            self.children: List[Leaf] = []

        def __str__(self):
            return str(self.data)

        def append(self, Node):
            pass


    root = Tree()
    root.data = "root"
    root.left = Tree()
    root.left.data = "left"
    root.right = Tree()
    root.right.data = "right"

    print(root.left.data)


    root = Tree()
    root.data = "root"
    root.left = Tree()
    root.left.data = "left"
    root.right = Tree()
    root.right.data = "right"

    root.left.left = Tree()
    root.left.left.data = "left 2"
    root.left.right = Tree()
    root.left.right.data = "left-right"

    print(root.data)
    print(root.left.left.data)

    # Reference: https://pythonspot.com/python-tree/


    # no longer binary ...
    class Tree(object):
        def __init__(self):
            self.left = None
            self.child = []
            self.data = []

        def createChildren(self, amount):
            for i in range(0, amount):
                self.child.append(Tree())

        def setChildrenValues(self, list):
            for i in range(0, len(list)):
                self.data.append(list[i])


    root = Tree()
    root.createChildren(3)
    root.setChildrenValues([5, 6, 7])
    root.child[0].createChildren(2)
    root.child[0].setChildrenValues([1, 2])
    # print some values in the tree
    print(root.data[0])
    print(root.child[0].data[0])
    """



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
""" implore.py - An even more desperate way to make requests for Humans.™ """
""" GOALS:
    By creating this app, I wanted to answer these questions:

    - What are the internet activities that I use or want most often?
    - What are the things that are feasible, but that I cannot easily do?

    Here is a list of items I came up with. I have solved them to some degree, but any advice or suggestions is welcome. See Contributors document for information.

    - Get a webpage
    - Modify CSS
    - Convert to a 3d object
        - front is main page
        - shape / depth controlled by varying functions
    - Collect links, images, tags, or something else from webpages
    - Create a 'map' of link destinations
    - Create an image catalog for a specific site(with CSS)
    - Interact with underlying database
    - Disable certain JS apps
    - Disable or interact with certain trackers
    - Watch for cookies and other local data / permissions
    - Block specific activities of specific sites
    - Search for English (or language) words and separate them from tags
    - Analyze metadata
    - Count types of tags and metadata
    - Analyze or avoid ads
    - Test for API functionality
    """


if True:  # Standard Library Dependencies
    from collections import Counter, deque
    from datetime import datetime
    from pprint import pprint
    from time import time, sleep, process_time, perf_counter, gmtime, localtime
    from typing import Any, Dict, List, MutableSequence
    import cgitb
    import os
    import sys
    import traceback

_debug_: bool = True  # turn on for Dev Debugging
if _debug_:  # enable detailed logging
    # cgitb.enable([display[, logdir[, context[, format]]]])
    DEFAULT_CGITB_DISPLAY: int = 1  # 0 to suppress messages
    DEFAULT_CGITB_FILE_LOGDIR: str = os.path.join(
        os.path.dirname(__file__), "LOGS"
    )
    DEFAULT_CGITB_CONTEXT: int = 3  # 5 is the normal default
    DEFAULT_CGITB_FORMAT: str = "html"

    cgitb.enable(
        logdir=DEFAULT_CGITB_FILE_LOGDIR,
        display=DEFAULT_CGITB_DISPLAY,
        context=DEFAULT_CGITB_CONTEXT,
        format=DEFAULT_CGITB_FORMAT,
    )

    DEFAULT_CGITB_ERROR: sys.exc_info = ()

if True:  # External Dependencies
    from apscheduler.schedulers.blocking import BlockingScheduler
    from dev.debug import dbprint, lru_cache, NL
    from bs4 import BeautifulSoup
    import requests

    try:
        import ujson as json  # faster json if available
    except:
        import json

if True:  # setup html parser
    DEFAULT_PARSER: str = ""
    XML: bool = False  # whether XML parsing is available at all
    try:  # use <lxml> for speed (HTML and / or XML) (VERY fast)
        import lxml as parser

        DEFAULT_PARSER = "lxml"
        XML = True
        XML_PARSER: str = "xml"
    except:
        try:  # use <html5lib> for accuracy (VERY slow)
            import html5lib as parser

            DEFAULT_PARSER = "html5lib"
        except:  # use <html5lib> Python's built-in html parser
            import html.parser as parser

            DEFAULT_PARSER = "html.parser"

if True:  # Default constants
    # * text messaging
    DEFAULT_SCRIPT_PATH: str = os.getcwd() + "/sentMessage.scpt"
    DEFAULT_CELL_NUMBER: str = "13616488261"

    # * time zones and scheduling
    USE_TZ: bool = False
    sch = BlockingScheduler()

    # * html and parsing
    DEFAULT_ENCODING: str = "UTF-8"
    url: str = ""
    script_text: str = ""
    current_number: str

    now: struct_time = gmtime()


class Contact_List(list):
    pass
    # joe = Contact("Joe", "+555555555", "Somewhere Else")


def db_print(*args, file=sys.stderr, **kwargs):
    """ Print messages to STDERR in debug mode (if <_debug_> is set) """
    if _debug_:
        print("@db ->", *args, **kwargs)


# contact_list.save_to_file() # with open('list.txt') as p: # p = json.load(p) # print(p)


class Error(Exception):
    """ Custom Exception handler for Implore. """

    def __init__(self, *args, **kwargs):
        self.args = args
        Exception.__init__(self, *args, **kwargs)
        return


class WebPageError(TypeError):
    """  # Exception raised for errors in the WebPage class. """

    pass


class WebPage(requests.Response):
    """ A local representation of a webpage. This object is able to analyze itself, share data, and perform its duties unsupervised.

    parameters

    url: the source of the original page grab. Use the <refresh> command to get an updated version."""

    # >>> r = requests.get('https://api.github.com/user',

    def __init__(self, url):
        super().__init__()
        self.url: str = url
        self.dirty: bool = True
        self.last_status: int = 0
        self.timestamp: struct_time = gmtime()
        # self.tags = {}
        # self.auth: Tuple[str, str] = ('user', 'pass')
        # auth = ('user', 'pass')

    def get_url_content(self, url: str) -> str:
        """ Return decoded contents from <url> using default parameters. """
        self = requests.get(url)
        self.history.append()
        self.last_status = self.status_code
        if self.last_status == 200:
            return self.text
        else:
            return ""

    def tag_find(
        self,
        tag_name: str,
        attrs_pass: Dict[Any, Any],
        parser_pass: str = DEFAULT_PARSER,
    ) -> List[Any]:
        """ Find matching tags from url. """
        soup = BeautifulSoup(self.text, features=DEFAULT_PARSER)
        return soup.findAll(name=tag_name, attrs=attrs_pass)

    def tag_list(self, tag_name: str):
        if self.dirty:
            self.tags = Counter(self.text)
            return self.tags

    def to_markdown():
        pass

    def to_json(self):
        return json.dumps(self.text)

    def stats(self):
        pass

    def soup(self):
        pass  # return BeautifulSoup()


class WebPageSet(deque):
    """ Stores and maintains a set of web pages. """

    DEFAULT_WEBPAGESET_SIZE = 2000  # maximum number of pages
    # TODO this should be 'maximum size' ... not count
    # ... and class should check it's own size

    def __init__(
        self,
        iterable: MutableSequence,
        maxlen: int = 0,
        check_links: bool = True,
        image_storage: str = "",
    ):
        if not maxlen or maxlen < 1:
            maxlen = DEFAULT_WEBPAGESET_SIZE
        self.check_links = check_links
        self.store = True if image_storage else False
        self.image_storage = image_storage
        super().__init__(iterable, maxlen)

    def count(self):
        """ count tags, emails, ... whatever from the entire set """
        pass

    def common(self):
        """ find items that this pageset has in common. """
        pass

    def append(self, x):
        if isinstance(x, WebPage):
            super().append(x)
        else:
            if _debug_:
                db_print()
            else:
                raise (WebPageError)

    def size_check(self):
        print(self.__sizeof__())


class Text_Messaging:
    def text(
        message: str, cell: str = DEFAULT_CELL_NUMBER, verbose: bool = False
    ) -> int:
        """ Return status of {message} sent to {cell}(verbose - give CLI feedback if True """
        script_text = "osascript {} {} '{}'".format(
            DEFAULT_SCRIPT_PATH, cell, message
        )
        db_print("script_text: ", script_text)
        try:
            result = os.system(script_text)
        except:
            result = -1
        finally:
            db_print("result: ", result)
            return result

    def test_text(
        self, message: str = "A python program just sent you a message ..."
    ):
        """ Send a test message using <text> """
        test_cell_number = DEFAULT_CELL_NUMBER
        db_print(f"{message=}")
        result = text(message=message, verbose=_debug_)
        print(f"{result=}")


@app.route("/")
def index():
    pass


def main_test():
    url = "https://www.indeed.com/jobs?q=python&l=Remote"
    matches = soup_match(url, tag_name="div", attrs_pass={"class": "title"})
    # for jobTitle in matches: # if "Developer" in jobTitle.text:
    # text(default_cell_number, 'jobTitle.text')  # break # elif "Jr" in jobTitle.text:
    # print(type(matches)) for match in matches: print(match.text.strip())
    # text(default_cell_number, 'jobTitle.text')  # break


def main():
    db_print(f"{DEFAULT_PARSER}")
    wps = WebPageSet([], check_links=True, maxlen=100)
    wps.append(WebPage("https://www.google.com"))
    wps.append(WebPage("fake page"))
    wps.append("fake page")


if __name__ == "__main__":
    cgitb.enable()
    db_print(cgitb.grey("test grey font"))
    try:
        app.run()
    except:
        pass

    # start_response('200 OK', [('Content-Type', 'text/html')])
    main()


# References
""" Choice of parser

    I use BeautifulSoup to work with html documents.

        Beautiful Soup is a Python library for pulling data out of HTML and XML files. It works with your favorite parser to provide idiomatic ways of navigating, searching, and modifying the parse tree. It commonly saves programmers hours or days of work.

        https://www.crummy.com/software/BeautifulSoup/bs4/doc/

    According to the documentation:

        $ pip install lxml
        $ pip install html5lib

    do not use python's built in html parser (before python 3.2.2).

    - "lxml" - Use for speed (HTML)
    - "xml" - Use for speed (XML)
    - "html5lib" - Best accuracy ... very slow! (HTML)
    - "html.parser" - built in parser ... eh, it works (HTML)
    - lxml is also the ONLY supported XML parser


Latex / Bibtex

    - http://www.bibtex.org/
    - https://libguides.bates.edu/thesis_writers

Shields

    - https://shields.io/
    - https://simpleicons.org/?q=dj
    - https://docs.readthedocs.io/en/stable/badges.html
    - https://img.shields.io/badge/Django-v3-%23092E20?logo=django&color=#339933


    """



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2007-2018 The Python-Twitter Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utility to get your access tokens."""

# 'future imports'
from __future__ import print_function

# 'Standard Library'
import sys
import webbrowser

# 'package imports'
from requests_oauthlib import OAuth1Session

if sys.version_info.major < 3:
    input = raw_input

REQUEST_TOKEN_URL = "https://api.twitter.com/oauth/request_token"
ACCESS_TOKEN_URL = "https://api.twitter.com/oauth/access_token"
AUTHORIZATION_URL = "https://api.twitter.com/oauth/authorize"
SIGNIN_URL = "https://api.twitter.com/oauth/authenticate"


def get_access_token(consumer_key, consumer_secret):
    """Get an access token for a given consumer key and secret.

    Args:
        consumer_key (str):
            Your application consumer key.
        consumer_secret (str):
            Your application consumer secret.

    Returns:
        (None) Prints to command line.
    """
    oauth_client = OAuth1Session(
        consumer_key, client_secret=consumer_secret, callback_uri="oob"
    )

    print("\nRequesting temp token from Twitter...\n")

    resp = oauth_client.fetch_request_token(REQUEST_TOKEN_URL)

    url = oauth_client.authorization_url(AUTHORIZATION_URL)

    print(
        "I will try to start a browser to visit the following Twitter page "
        "if a browser will not start, copy the URL to your browser "
        "and retrieve the pincode to be used "
        "in the next step to obtaining an Authentication Token: \n"
        "\n\t{0}".format(url)
    )

    webbrowser.open(url)
    pincode = input("\nEnter your pincode? ")

    print("\nGenerating and signing request for an access token...\n")

    oauth_client = OAuth1Session(
        consumer_key,
        client_secret=consumer_secret,
        resource_owner_key=resp.get("oauth_token"),
        resource_owner_secret=resp.get("oauth_token_secret"),
        verifier=pincode,
    )
    try:
        resp = oauth_client.fetch_access_token(ACCESS_TOKEN_URL)
    except ValueError as e:
        raise "Invalid response from Twitter requesting temp token: {0}".format(
            e
        )

    print(
        """Your tokens/keys are as follows:
        consumer_key         = {ck}
        consumer_secret      = {cs}
        access_token_key     = {atk}
        access_token_secret  = {ats}""".format(
            ck=consumer_key,
            cs=consumer_secret,
            atk=resp.get("oauth_token"),
            ats=resp.get("oauth_token_secret"),
        )
    )


def main():
    """Run script to get access token and secret for given app."""
    consumer_key = input("Enter your consumer key: ")
    consumer_secret = input("Enter your consumer secret: ")
    get_access_token(consumer_key, consumer_secret)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

# 'Standard Library'
import os

from functools import lru_cache
from pathlib import Path

from typing import List

# root search path
search_path: str = Path().cwd().as_posix()


# !-------------------- Target Extensions list
# We only are interested in these file types
targets = ["html", "txt"]

# !-------------------- Exclusions list
# If the file path contains these we dont want them
#    eg. C:\\Directory\\ForWalking\\node_modules will be ignored
exclusions = ["node_modules", "SolutionFiles", ".bin", "Test"]


# @lru_cache
def get_list(
    search_path: str = search_path,
    targets: List[str] = targets,
    exclusions: List[str] = exclusions,
    recursive: bool = True,
):
    """ #### Return a list of files

        - <search_path> - root path for recursive search
        - <targets> - list of globs to locate
        - <exclusions> - list of globs to exclude
        - <recursive> -
        """

    # Array to store all our file paths
    file_paths = []

    for root, sub_dirs, file_names in os.walk(search_path):

        # Check each file against the <targets>
        #   and <exclusions> lists
        for file_name in file_names:

            # If the file path includes files with extensions
            #   from the targets list ...
            if any(target in root for target in targets):

                # ... and if the file path doesnt have the exclusion list,
                #   add it to the file_paths list
                if not any(exclusion in root for exclusion in exclusions):
                    file_paths.append(os.path.join(root, file_name))


file_paths: List[str] = get_list()
print(file_paths)



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Author: Michael Treanor
# Contact: skeptycal@gmail.com
# Copyright: 2020 Michael Treanor
# License: MIT
""" join - Join pages from a collection of PDF files into a single PDF file.

    Usage:
        join FILES [--output <file>] [--shuffle] [--verbose]

    Parameters:
        --shuffle
            Take a page from each PDF input file in turn before
            taking another from each file. If this option is not
            specified then all of the pages from a PDF file are
            appended to the output PDF file before the next input
            PDF file is processed.

        --verbose
            Write information about the progress of this tool to stderr.

    # modified version of code from macOS Automator (macOS 10.15.4)
    /System/Library/Automator/Combine\ PDF\ Pages.action/Contents/Resources/join.py

    """

__docformat__ = "reStructuredText"

std_inp = "AREPL requires standard_input to be hardcoded, like so: standard_input = 'hello world'; print(input())"

for d in dir():
    print(d)
    print("-" * 60)
    try:
        print(eval(d))
    except:
        print(eval(d + "()"))
    print("-" * 60)

# print(__doc__)

"""
import sys
import os
import getopt
import tempfile
import shutil
from CoreFoundation import *
from Quartz.CoreGraphics import *

verbose = False


def createPDFDocumentWithPath(path):
    global verbose
    if verbose:
        print("Creating PDF document from file {}".format(path))
    return CGPDFDocumentCreateWithURL(CFURLCreateFromFileSystemRepresentation(kCFAllocatorDefault, path, len(path), False))


def writePageFromDoc(writeContext, doc, pageNum):
    global verbose
    page = CGPDFDocumentGetPage(doc, pageNum)
    if page:
        mediaBox = CGPDFPageGetBoxRect(page, kCGPDFMediaBox)
    if CGRectIsEmpty(mediaBox):
        mediaBox = None

    CGContextBeginPage(writeContext, mediaBox)
    CGContextDrawPDFPage(writeContext, page)
    CGContextEndPage(writeContext)
    if verbose:
        print("Copied page %d from %s" % (pageNum, doc))


def shufflePages(writeContext, docs, maxPages):

    for pageNum in xrange(1, maxPages + 1):
        for doc in docs:
            writePageFromDoc(writeContext, doc, pageNum)


def append(writeContext, docs, maxPages):

    for doc in docs:
        for pageNum in xrange(1, maxPages + 1):
            writePageFromDoc(writeContext, doc, pageNum)


def main(argv):

    global verbose

    # The PDF context we will draw into to create a new PDF
    writeContext = None

    # If True then generate more verbose information
    source = None
    shuffle = False

    # Parse the command line options
    try:
        options, args = getopt.getopt(argv, "o:sv", ["output=", "shuffle", "verbose"])
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    for option, arg in options:
        if option in ("-o", "--output"):
            if verbose:
                print("Setting %s as the destination." % (arg))
            writeContext = CGPDFContextCreateWithURL(CFURLCreateFromFileSystemRepresentation(
                    kCFAllocatorDefault, arg, len(arg), False), None, None)

        elif option in ("-s", "--shuffle"):
            if verbose:
                print("Shuffle pages to the output file.")
            shuffle = True

        elif option in ("-v", "--verbose"):
            print("Verbose mode enabled.")
            verbose = True

        else:
            print("Unknown argument: %s" % (option))

    if writeContext:
        # create PDFDocuments for all of the files.
        docs = map(createPDFDocumentWithPath, args)

    # find the maximum number of pages.
    maxPages = 0
    for doc in docs:
        if CGPDFDocumentGetNumberOfPages(doc) > maxPages:
            maxPages = CGPDFDocumentGetNumberOfPages(doc)

    if shuffle:
        shufflePages(writeContext, docs, maxPages)
    else:
        append(writeContext, docs, maxPages)

    CGPDFContextClose(writeContext)
    del writeContext
    #CGContextRelease(writeContext)


def usage():
    print("Usage: join FILES [--output <file>] [--shuffle] [--verbose]")


if __name__ == "__main__":
    main(sys.argv[1:])
"""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# 'Standard Library'
from pprint import pformat

# 'package imports'
import text_utils

from text_utils import *


@dataclass
class Ascii_Chars(dict):
    indent: int = 2
    ascii_range: List[int] = range(0, 255)
    indent_char: str = " "

    def __post_init__(self, indent=2):
        self.ind: str = self.indent_char * self.indent
        self.update({f"c{x}": chr(x) for x in self.ascii_range if chr(x)})

    def __str__(self, indent: int = 2) -> (str):
        return pformat(self.items(),indent=self.indent, depth=5)
        return ", ".join(
        [f"{self.ind}{x}" for x in self.to_list()]
    )

    #? ------------------------ contents checks

    def is_printable(self, value):
        return str(value).isprintable

    def is_alpha(self, value):
        return str(value).isalpha

    def is_alphanum(self, value):
        return str(value).isalnum

    def is_hex(self, value):
        return value in STR_HEX

    def is_name(self, value):
        return str(value).isidentifier

    def is_punc(self, value):
        return value in STR_PUNCTUATION

    def is_white(self, value):
        return str(value).isspace

    def is_ascii(self, value):
        return str(value).isascii

    def is_dec(self, value):
        return str(value).isdecimal

    def is_digit(self, value):
        return str(value).isdigit

    def is_lower(self, value):
        return str(value).islower

    def is_upper(self, value):
        return str(value).isupper

    def is_title(self, value):
        return str(value).istitle

    #? ------------------------ conversions

    def to_hex(self,value):
        return f"{value:x}"

    def to_octal(self,value):
        return f"{value:o}"

    def to_sci(self,value):
        return f"{value:e}"

    def to_title(self,value):
        return str(value).title

    #? ------------------------ type conversions

    def to_list(self):
        # return self.values()
        return [f"{k}: {v}" for k, v in self.items() if self.is_printable(v)]

    #? ------------------------ formatting

    def to_columns(self, n: int = 3, width: int = 80):
        items = self.to_list()
        print(items)
        size: int = len(items)
        print(size)
        size_per: int = size // n
        print(size_per)
        result = []
        for i, x in enumerate(items):

            result = NL.join([f"{x:>15.15}     |     {x:>15.15}" for x in items])
        print(result)


print(Ascii_Chars())

a = Ascii_Chars()
print(repr(a))
print(str(a))
# a.to_columns()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" Part of the `AutoSys` package
        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """


# 'Standard Library'
# !---------------------------------------------- Imports
# from autosys import *
import pathlib

# 'package imports'
from autosys.debug import db_column_ruler, dbprint

# !---------------------------------------------- CONSTANTS
_debug_: bool = True  # True => use Debug features
SCRIPT_PATH: str = pathlib.Path(__file__).resolve().parents[0].as_posix()
PARENT_PATH: str = pathlib.Path(__file__).resolve().parents[1].as_posix()

# !---------------------------------------------- String Utilities


def set_script_in_sys_path():
    """ Put this script in the system path if it is not. """
    dbprint(f"SCRIPT_PATH: {SCRIPT_PATH}")
    dbprint(f"PARENT_PATH: {PARENT_PATH}")
    br()

    if SCRIPT_PATH not in sys.path:
        sys.path.insert(0, SCRIPT_PATH)
        try:  # skip debug output if not available
            dbprint(f"SCRIPT_PATH *NOT* in sys.path.")
        except:
            pass
    else:
        try:  # skip debug output if not available
            dbprint(f"SCRIPT_PATH *IS* in sys.path.")
        except:
            pass


def _tests_():
    """ Debug Tests for script. """
    br()

    db_column_ruler(8)
    br()

    dbprint(f"Rows: {ROWS} x Columns:{COLS}")
    br()

    dbprint(
        f"Python version (sys.version_info): {sys.version_info.major}.{sys.version_info.minor}"
    )
    dbprint("Script version (__version__):     ", __version__)
    br()

    dbprint(f"SYS.PATH: ")
    for p in sys.path:
        dbprint(f"    {p}")
    br()


def main():
    """
    CLI script main entry point.
    """
    set_script_in_sys_path()
    if _debug_:
        _tests_()


if __name__ == "__main__":  # if script is loaded directly from CLI
    main()



import re

try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension

with open("auto_loguru/__init__.py", "r") as file:
    regex_version = r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]'
    version = re.search(regex_version, file.read(), re.MULTILINE).group(1)

with open("README.rst", "r") as file:
    readme = file.read()

setup(
    name="auto_loguru",
    version=version,
    packages=["auto_loguru"],
    package_data={"auto_loguru": ["__init__.pyi", "py.typed"]},
    description="Fancy defaults for the awesome loguru logs!",
    # long_description=readme,
    # long_description_content_type="text/x-rst",
    author="Michael Treanor",
    author_email="skeptycal@gmail.com",
    url="https://github.com/skeptycal/auto_loguru",
    download_url="https://github.com/skeptycal/auto_loguru/archive/{}.tar.gz".format(
        version
    ),
    project_urls={
        "Changelog": "https://github.com/skeptycal/auto_loguru/blob/master/CHANGELOG.rst",
        "Documentation": "https://loguru.readthedocs.io/en/stable/index.html",
    },
    keywords=["loguru", "logging", "logger", "log"],
    license="MIT license",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: System :: Logging",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
    install_requires=[
        "colorama>=0.3.4 ; sys_platform=='win32'",
        "aiocontextvars>=0.2.0 ; python_version<'3.7'",
        "win32-setctime>=1.0.0 ; sys_platform=='win32'",
        "loguru>=0.5.0",
    ],
    extras_require={
        "dev": [
            "black>=19.10b0 ; python_version>='3.6'",
            "codecov>=2.0.15",
            "colorama>=0.3.4",
            "flake8>=3.7.7",
            "isort>=5.1.1 ; python_version>='3.6'",
            "tox>=3.9.0",
            "tox-travis>=0.12",
            "pytest>=4.6.2",
            "pytest-cov>=2.7.1",
            "Sphinx>=2.2.1",
            "sphinx-autobuild>=0.7.1",
            "sphinx-rtd-theme>=0.4.3",
        ]
    },
    python_requires=">=3.6",
)



# -*- coding: utf-8 -*-
""" Part of the `AutoSys` package
        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """
__license__ = "MIT"

__all__ = [
    "DEFAULT_SCRIPT_PATH",
    "DEFAULT_CELL_NUMBER",
    "TextMessaging",
    "TextMessagingError",
]

# 'Standard Library'
import os

# * text messaging

DEFAULT_SCRIPT_PATH: str = os.getcwd() + "./sentMessage.scpt"
# sendMessage.scpt
DEFAULT_SCRIPT: str = """on run {targetPhoneNumber, targetMessageToSend}
    tell application "Messages"
        set targetService to 1st service whose service type = iMessage
        set targetBuddy to buddy targetPhoneNumber of targetService
        set targetMessage to targetMessageToSend
        send targetMessage to targetBuddy
    end tell
end run"""

DEFAULT_CELL_NUMBER: str = "xxxxxxxxxxx"

with open("cell_number_private.txt", mode="r") as f:
    DEFAULT_CELL_NUMBER: str = f.readline().strip("()-")
    print(DEFAULT_CELL_NUMBER)


class TextMessagingError(Exception):
    pass


class TextMessaging:
    def __init__(
        self, number=DEFAULT_CELL_NUMBER, script_path=DEFAULT_SCRIPT_PATH
    ):
        self.number = number
        self.script_path = script_path

    def text(self, message: str, verbose: bool = False) -> int:
        """ Return status of {message} sent to {cell}(verbose - give CLI feedback if True """
        script_text = "osascript {} {} '{}'".format(
            DEFAULT_SCRIPT_PATH, self.number, message
        )
        db_print("script_text: ", script_text)
        try:
            result = os.system(script_text)
        except:
            result = -1
        finally:
            db_print("result: ", result)
            return result

    def test_text(
        self, message: str = "A python program just sent you a message ..."
    ):
        """ Send a test message using <text> """
        db_print(f"{message=}")
        result = self.text(message=message, verbose=True)
        print(f"{result=}")


tm = TextMessaging()



# 'Standard Library'
from os import linesep as NL
from sys import stderr, stdout

from typing import List

try:
    _debug_
except:
    _debug_: bool = True

_DEBUG_COLOR: str = "\x1B[38;5;178m"  # private ansi CLI color code
_RESET: str = "\x1B[0m"  # private ansi CLI reset code


def arg_str(*data, arg_sep: str = "", **kwargs):
    """ Return string from an iterable of variously typed arguments. Each item is converted separately so

        data: iterable - iterable containing items
        sep: str  - separator between list items

        Errors in types and formatting are ignored.
        """
    tmp: List[str] = []
    if len(data) == 0 or data == None:
        return ""

    # weed out values that do not convert to str cleanly
    for item in data:
        try:
            tmp.append(str(item))
        except (AttributeError, TypeError, ValueError, IndexError) as e:
            pass  # ignore errors

    # convert list to str and return
    try:
        return arg_sep.join(data)
    except (AttributeError, TypeError, ValueError, IndexError) as e:
        return ""  # return valid string even if errors present


def dbprint(*db_args, sep=" ", end=NL, file=stderr, flush=False):
    """ Prints debug messages if _debug_ flag is True.

        The default <sep> and <end> are empty strings which allows for easier custom formatting. Change the defaults as needed.

        Example:
        ```py
        if arg == '--version':
            dbprint(f"anansi.py version {Ansi.lime}{__version__}.")
        ```
        """
    if _debug_:
        # PREFIX    args    SUFFIX   Newline
        print(
            f"{_DEBUG_COLOR}{arg_str(*db_args)}{_RESET}",
            sep="",
            end=end,
            file=file,
            flush=flush,
        )


if __name__ == "__main__":
    dbprint("This stuff is a test...")
